Package: klassR
Type: Package
Title: Classifications for Statistics Norway
Version: 1.0.6
Authors@R: 
  c(person(given = "Susie",
           family = "Jentoft",
           role = "aut"),
    person(given = "Diana-Cristina",
           family = "Iancu",
           role = "aut"),
    person(given = "Lisa",
           family = "Li",
           role = "aut"),
    person(given = "Øyvind I.",
           family = "Berntsen",
           role = c("aut", "cre"),
           email = "Oyvind.Berntsen@ssb.no"),
    person("Statistics Norway", role = "cph"))
Description: Functions to search, retrieve, apply and update classification 
  standards and code lists using Statistics Norway's API 
  <https://www.ssb.no/klass> from the system 'KLASS'. Retrieves classifications 
  by date with options to choose language, hierarchical level and formatting.
Depends: R (>= 3.5.0)
Imports: tm, httr, jsonlite, igraph (>= 2.1.1), methods
URL: https://statisticsnorway.github.io/ssb-klassr/
BugReports: https://github.com/statisticsnorway/ssb-klassr/issues
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Suggests: rmarkdown, knitr, testthat (>= 3.0.0), kableExtra, magrittr,
        dplyr
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-15 13:30:58 UTC; onyxia
Author: Susie Jentoft [aut],
  Diana-Cristina Iancu [aut],
  Lisa Li [aut],
  Øyvind I. Berntsen [aut, cre],
  Statistics Norway [cph]
Maintainer: Øyvind I. Berntsen <Oyvind.Berntsen@ssb.no>
Repository: CRAN
Date/Publication: 2026-06-16 06:50:20 UTC
Built: R 4.5.2; ; 2026-06-16 14:06:40 UTC; unix

# Generated by roxygen2: do not edit by hand

export(ApplyKlass)
export(GetKlass)
export(apply_klass)
export(correspond_list)
export(get_family)
export(get_klass)
export(get_name)
export(get_version)
export(klass_graph)
export(klass_node)
export(list_family)
export(list_klass)
export(search_klass)
export(update_klass)
export(update_klass_node)

#' Get base address for fetching classifications from
#' @keywords internal
#' @return String with address
GetBaseUrl <- function() {
  "https://data.ssb.no/api/klass/v1/"
}

#' Get target ID numbers from Url
#'
#' @param x Url address
#' @keywords internal
#' @return Number
GetNums <- function(x) {
  x <- as.character(x)
  gsub(".*/", "", x)
}

#' Get json file from Url
#'
#' @param url String url address
#' @keywords internal
#' @return text in json format
GetUrl <- function(url) {
  hent_klass <- check_connect(url)
  if (is.null(hent_klass)) {
    return(invisible(NULL))
  } else {
    klass_text <- httr::content(hent_klass, "text", encoding = "UTF-8") ## deserialisering med httr funksjonen content
    klass_data <- jsonlite::fromJSON(klass_text)
    return(klass_data)
  }
}


#' Classification list
#' Get a full list of all classifications and codelists
#'
#' @param codelists True/False for whether to include codelists. Default = FALSE
#' @param language Two letter string for the requested language output. Default is Bokmål ("nb"). Nynorsk ("nn") and English ("en").
#'
#' @return A data frame containing a full list of classifications. The data frame includes the classification name, number, family and type.
#' @export
#'
#' @examples
#' head(list_klass(codelists = TRUE))
list_klass <- function(codelists = FALSE, language = "nb") {
  fams <- list_family()$family_nr
  Klist <- data.frame(klass_name = NA, klass_nr = NA, klass_family = NA, klass_type = NA)

  # create code for including codelists and language
  code <- ifelse(codelists, "?includeCodelists=true", "")
  code <- ifelse(code == "", paste0(code, "?language=", language),
    paste0(code, "&language=", language)
  )

  for (i in fams) {
    url <- paste(GetBaseUrl(), "classificationfamilies/", i, code, sep = "")
    dt <- data.frame(GetUrl(url)$classifications)
    nums <- as.vector(sapply(dt$X_links[, 1], GetNums))
    if (nrow(dt) > 0) {
      dt2 <- data.frame(klass_name = dt$name, klass_nr = nums, klass_family = i, klass_type = dt$classificationType)
      Klist <- rbind(Klist, dt2)
    }
  }
  return(Klist[-1, ])
}

#' @rdname list_klass
ListKlass <- function(codelists = FALSE, language = "nb") {
  .Deprecated("list_klass")
  list_klass(codelists = codelists, language = language)
}


#' Classification family list
#' Print a list of all families and the number of classifications in each
#'
#' @param family Input family ID number to get a list of classifications in that family
#' @param codelists True/False for whether to include codelists. Default = FALSE
#' @param language Two letter string for the requested language output. Default is Bokmål ("nb"). Nynorsk ("nn") and English ("en").
#' @return dataset containing a list of families
#' @export
#'
#' @examples
#' list_family(family = 1)
list_family <- function(family = NULL, codelists = FALSE, language = "nb") {
  # create code for including codelists and language
  code <- ifelse(codelists, "?includeCodelists=true", "")
  code <- ifelse(code == "", paste0(code, "?language=", language),
    paste0(code, "&language=", language)
  )

  # If no family specified then show all families
  if (is.null(family)) {
    url <- paste(GetBaseUrl(), "classificationfamilies", code, sep = "")
    dt <- data.frame(GetUrl(url)$"_embedded"$classificationFamilies)
    nums <- as.vector(sapply(dt$X_links$self$href, FUN = GetNums))
    dt2 <- data.frame(
      family_name = dt$name, family_nr = nums,
      number_of_classifications = dt$numberOfClassifications
    )
  }

  # If a family is given the show classifications within that family
  if (!is.null(family)) {
    family <- MakeChar(family)
    url <- paste(GetBaseUrl(), "classificationfamilies/", family, code, sep = "")
    dt <- data.frame(GetUrl(url)$classifications)
    nums <- as.vector(sapply(dt$X_links[, 1], GetNums))
    dt2 <- data.frame(klass_name = dt$name, klass_nr = nums)
    row.names(dt2) <- NULL
  }
  return(dt2)
}


#' @rdname list_family
ListFamily <- function(family = NULL, codelists = FALSE, language = "nb") {
  .Deprecated("list_family")
  list_family(family = family, codelists = codelists, language = language)
}


#' Search Klass
#'
#' @param query String with key word to search for
#' @param codelists True/False for whether to include codelists. Default = FALSE
#' @param size The number of results to show. Default = 20.
#'
#' @return Data frame of possible classifications that match the query
#' @export
#'
#' @examples
#' search_klass("occupation")
search_klass <- function(query, codelists = FALSE, size = 20) {
  query <- as.character(query)
  code <- ifelse(codelists, "&includeCodelists=true", "")
  url <- paste(GetBaseUrl(), "classifications/search?query=", query, code, "&size=", size, sep = "")
  dt <- data.frame(GetUrl(url)$"_embedded"$searchResults)
  nums <- as.vector(sapply(dt$X_links$self$href, GetNums))
  dt2 <- data.frame(klass_name = dt$name, klass_nr = nums)
  row.names(dt2) <- NULL
  return(dt2)
}


#' @rdname search_klass
SearchKlass <- function(query, codelists = FALSE, size = 20) {
  .Deprecated("search_klass")
  search_klass(query = query, codelists = codelists, size = size)
}


#' Get version number of a class given a date
#'
#' @param classification Classification number
#' @param date Date for version to be valid
#' @param family Family ID number if a list of version number for all classes is desired
#' @param klassNr True/False for whether to output classification numbers. Default = FALSE
#'
#' @return Number, vector or data frame with version numbers and calssification numbers if specified.
#' @export
#'
#' @examples
#' get_version(7)
get_version <- function(classification = NULL, date = NULL, family = NULL, klassNr = FALSE) {
  if (is.null(date)) date <- Sys.Date()
  if (is.null(family)) {
    if (klassNr == TRUE) stop("To output classification number from this function you need to input a family number")
    classification <- MakeChar(classification)
    url <- paste(GetBaseUrl(), "classifications/", classification, sep = "")
    df <- as.data.frame(GetUrl(url)$versions)
    df$validTo[is.na(df$validTo)] <- as.character(Sys.Date() + 1)
    for (i in 1:nrow(df)) {
      cond <- as.Date(date) >= as.Date(df$validFrom[i]) & as.Date(date) < as.Date(df$validTo[i])
      if (cond) {
        vers <- GetNums(df$`_links`$self$href[i])
      }
    }
  } else {
    family <- MakeChar(family)
    fam <- list_family(family, codelists = TRUE)
    vers <- NULL
    klass_nr <- NULL
    for (i in fam$klass_nr) {
      url <- paste(GetBaseUrl(), "classifications/", i, sep = "")
      df <- as.data.frame(GetUrl(url)$versions)
      if (length(df) == 0) next() # Check if there is a valid version number
      if (is.null(df$validTo)) df$validTo <- as.character(Sys.Date() + 1)
      df$validTo[is.na(df$validTo)] <- as.character(Sys.Date() + 1)
      for (j in 1:nrow(df)) {
        cond <- as.Date(date) >= as.Date(df$validFrom[j]) & as.Date(date) < as.Date(df$validTo[j])
        if (cond) {
          vers <- c(vers, GetNums(df$`_links`$self$href[j]))
          klass_nr <- c(klass_nr, i)
        }
      }
    }
    if (klassNr == TRUE) {
      vers <- data.frame(vers, klass_nr)
    }
  }
  return(vers)
}


#' @rdname get_version
#' @param klass Deprecated; use `classification` instead.
GetVersion <- function(klass = NULL, date = NULL, family = NULL, klassNr = FALSE) {
  .Deprecated("get_version")
  get_version(classification = klass, date = date, family = family, klassNr = klassNr)
}


#' Get the name of a classification version
#'
#' @param version Version number
#'
#' @return string or vector of strings with name of version
#' @export
#'
#' @examples
#' get_name("33")
get_name <- function(version) {
  version <- MakeChar(version)
  vernames <- NULL
  for (i in version) {
    url <- paste(GetBaseUrl(), "versions/", i, sep = "")
    vernames <- c(vernames, GetUrl(url)$name)
  }
  return(vernames)
}


#' @rdname get_name
GetName <- function(version) {
  .Deprecated("get_name")
  get_name(version = version)
}


#' Identify corresponding family from a classification number
#'
#' @param classification Classification number
#'
#' @return Family number
#' @export
#'
#' @examples
#' get_family(classification = 7)
get_family <- function(classification) {
  classification <- MakeChar(classification)
  K <- list_klass(codelists = TRUE)
  m <- match(classification, K$klass_nr)
  return(K$klass_family[m])
}


#' @rdname get_family
#' @param klass Deprecated; use `classification` instead.
GetFamily <- function(klass) {
  .Deprecated("get_family")
  get_family(classification = klass)
}


#' Correspondence list
#' Print a list of correspondence tables for a given classification with source and target IDs
#'
#' @param classification Classification number
#' @param date Date for classification (format = "YYYY-mm-dd"). Default is current date
#'
#' @return Data frame with list of corrsepondence tables, source ID and target ID.
#' @export
#'
#' @examples
#' \donttest{
#' correspond_list("7")
#' }
correspond_list <- function(classification, date = NULL) {
  cat("Finding correspondence tables ...")
  classification <- MakeChar(classification)
  if (is.null(date)) {
    date <- Sys.Date()
  }
  vers <- get_version(classification = classification, date = date)
  url <- paste(GetBaseUrl(), "versions/", vers, sep = "")
  df <- GetUrl(url)
  versName <- df$name
  dt <- data.frame(df$correspondenceTables)
  fam <- get_family(classification = classification)
  versValid <- get_version(family = fam, date = date, klassNr = TRUE)
  vers_names <- get_name(versValid$vers)
  source_klass <- NULL
  target_klass <- NULL

  for (i in 1:nrow(dt)) {
    m <- match(versName, c(dt$source[i], dt$target[i]))
    findName <- ifelse(m == 2, dt$source[i], dt$target[i])
    m2 <- match(findName, vers_names)
    newdate <- date
    counter <- 0
    while (is.na(m2) & counter < 10) { # hvis versjonen ikke ble funnet på date søkes det tilbake i tid
      newdate <- as.character(as.Date(newdate) - 60)
      versValidold <- get_version(family = fam, date = newdate, klassNr = TRUE)
      vers_names <- get_name(versValidold$vers)
      m2 <- match(findName, vers_names)
      counter <- counter + 1
      cat(".")
    }
    sourceTarget <- ifelse(is.na(m2), NA, as.character(versValid[m2, "klass_nr"]))
    source_klass[i] <- ifelse(m == 1, classification, sourceTarget)
    target_klass[i] <- ifelse(m == 1, sourceTarget, classification)
  }
  correspondence_table <- sapply(dt$X_links$self$href, GetNums)
  dt2 <- data.frame(
    correspondence_name = dt$name,
    source_klass = source_klass,
    target_klass = target_klass,
    correspondence_table, stringsAsFactors = FALSE
  )
  row.names(dt2) <- NULL
  dt2$target_klass[dt2$source_klass == dt2$target_klass] <- NA # dropping target for tables within version

  if (any(is.na(dt2$target_klass))) {
    message(
      "\n\n There are correspondence tables within classification ",
      classification,
      " (between different time points). Use the changes = TRUE option in the apply_klass and get_klass functions to get these\n "
    )
  }
  return(dt2)
}


#' @rdname correspond_list
#' @param klass Deprecated; use `classification` instead.
CorrespondList <- function(klass, date = NULL) {
  .Deprecated("correspond_list")
  correspond_list(classification = klass, date = date)
}

# Test searches for classifications

# library(klassR)

test_that("ListKlass returns a warning but still runs", {
  expect_warning(
    klass_table <- ListKlass()
  )
  expect_true(exists("klass_table"))
})


test_that("list_klass returns a list", {
  klass_table <- list_klass()
  expect_gt(nrow(klass_table), 1)

  codelist_table <- list_klass(codelists = TRUE)
  expect_gt(nrow(codelist_table), nrow(klass_table))
})


test_that("search_klass returns searches", {
  komm_search <- search_klass("kommune")
  expect_in("Standard for kommuneinndeling", komm_search$klass_name)
})

#' Given a Klass graph, find the node corresponding to a code and (optionally) a
#' date.
#'
#' @param graph A graph generated by \code{\link{klass_graph}}.
#' @param x The code to search for.
#' @param date Optional. The specific date the supplied code is valid in.
#'
#' @return The node in the graph corresponding to the supplied code. If date is
#'   not provided, the node with the most recent code is returned. If date is
#'   provided, the code with date between \code{validFrom} and \code{validTo} is
#'   returned.
#' @export
#'
#' @examples
#' \dontrun{
#' # Build a graph directed towards the most recent codes.
#' library(klassR)
#' klass_131 <- klass_graph(131)
#'
#' # Find the most recent node in the graph representing the code "0101" (Halden,
#' # valid to 2020.)
#' halden_node <- klass_node(klass_131, "0101")
#' }
klass_node <- function(graph, x, date = NA) {
  if (!is.na(date)) {
    date <- as.Date(date[[1]])

    node <- igraph::V(graph)[igraph::V(graph)$code == x &
      date >= igraph::V(graph)$validFrom &
      (date < igraph::V(graph)$validTo | is.na(igraph::V(graph)$validTo))]
  } else {
    x_indices <- which(igraph::V(graph)$code == x)

    highest_variant_index <-
      suppressWarnings(
        x_indices[which(igraph::V(graph)[x_indices]$variant ==
          max(igraph::V(graph)[x_indices]$variant))]
      )

    node <- igraph::V(graph)[highest_variant_index]
  }

  if (length(node) > 1) {
    stop("More than one node found.")
  } else {
    return(node)
  }
}

#' Count the neighbors of a node.
#'
#' @inheritParams update_klass_node
#' @inheritParams igraph::neighbors
#'
#' @return A numeric vector of length one giving the number of neighbors.
#'
#' @keywords internal
#'
count_neighbors <- function(graph, node, mode) {
  length(igraph::neighbors(graph, node, mode))
}

#' Given a graph and a node, determine if the node is a split code.
#'
#' @inheritParams update_klass_node
#'
#' @return \code{TRUE} if the node is split, otherwise \code{FALSE}.
#'
#' @details The function will attempt to reconcile nodes that have split and
#'   then later merged again. A node is considered to be split if there is more
#'   than one node that does not itself have children (i.e. nodes at the end of
#'   a sequence of changes) that can be reached from \code{node}
#'
#' @keywords internal
#'
is_split <- function(graph, node) {
  bfs_result <- igraph::bfs(
    graph = graph,
    root = node,
    mode = "out",
    unreachable = FALSE
  )

  end_nodes <- bfs_result$order[vapply(bfs_result$order,
    count_neighbors,
    graph = graph,
    mode = "out",
    FUN.VALUE = integer(1)
  ) == 0]

  length(unique(end_nodes)) > 1
}

#' Given a graph and a node, determine if the node is a result of combinations
#' of multiple codes.
#'
#' @inheritParams update_klass_node
#'
#' @param compare_node Optional. A node to compare \code{node} with when
#'   determining whether \code{node} is combined. See details.
#'
#' @return \code{TRUE} if the node is a combination of two or more nodes,
#'   otherwise \code{FALSE}.
#'
#' @details The function will attempt to reconcile nodes that have split and
#'   then later merged again when evaluating a node's combinedness.
#'
#'   If \code{compare_node == NULL}, a node is considered to be combined if more
#'   than one node that does not itself have a parent (i.e. codes at the start
#'   of a sequence of changes) contribute to \code{node}.
#'
#'   If \code{compare_node != NULL}, a node is considered to be combined if any
#'   node that is not an ancestor of \code{compare_node} contributes to
#'   \code{node}, i.e. all paths from \code{node} to the parents of \code{node}
#'   pass through \code{compare_node}.
#'
#' @keywords internal
#'
is_combined <- function(graph, node, compare_node = NULL) {
  bfs_result <- igraph::bfs(
    graph = graph,
    root = node,
    mode = "in",
    unreachable = FALSE
  )

  start_nodes <- bfs_result$order[vapply(bfs_result$order,
    count_neighbors,
    graph = graph,
    mode = "in",
    FUN.VALUE = integer(1)
  ) == 0]

  if (is.null(compare_node)) {
    length(unique(start_nodes)) > 1
  } else {
    paths <- igraph::all_simple_paths(graph, node, start_nodes, mode = "in")

    return(!all(vapply(paths, function(path) compare_node %in% path, logical(1))))
  }
}

#' Given a node and a graph, find the node at the end of a sequence of changes.
#'
#' @inheritParams klass_node
#' @param node A node as returned by \code{\link{klass_node}} or
#'   \code{\link[igraph]{V}}.
#'
#' @return A sequence of vertices, starting with \code{node} and ending with the
#'   last visited node.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' # Build a graph directed towards the most recent codes.
#' library(klassR)
#' klass_131 <- klass_graph(131)
#'
#' # Find the most recent node in the graph representing the code "0101" (Halden,
#' # valid to 2020.)
#' halden_node <- klass_node(klass_131, "0101")
#'
#' # Find the most recent code corresponding to 0101 Halden
#' halden_node_updated <- update_klass_node(klass_131, halden_node)
#' }
update_klass_node <- function(graph, node) {
  bfs_result <- igraph::bfs(
    graph = graph,
    root = node,
    mode = "out",
    unreachable = FALSE
  )


  end_nodes <- bfs_result$order[vapply(bfs_result$order,
    count_neighbors,
    graph = graph,
    mode = "out",
    FUN.VALUE = integer(1)
  ) == 0]

  visited <- unique(c(
    node,
    bfs_result$order[!bfs_result$order$name %in% unique(end_nodes)$name],
    end_nodes
  ))

  igraph::vertex_attr(graph, "split", visited$name) <-
    unname(vapply(visited, is_split, graph = graph, FUN.VALUE = logical(1)))

  igraph::vertex_attr(graph, "combined", visited$name) <-
    unname(vapply(visited,
      is_combined,
      graph = graph,
      compare_node = node,
      FUN.VALUE = logical(1)
    ))

  igraph::vertex_attr(graph, "nextNodes", visited$name) <-
    lapply(visited, igraph::neighbors, graph = graph, mode = "out")

  visited <- igraph::V(graph)[visited$name]

  return(visited)
}

test_that("is_combined correctly identifies combined nodes", {
  data(klass_131_graph)
  graph <- klass_131_graph

  ## Not using `compare_node`

  # Two parent nodes: 0103 Fredrikstad and 0113 Borge

  expect_true(is_combined(
    graph = graph,
    node = klass_node(graph, "0106", "1994-01-01")
  ))

  # One parent node through two edges: 0113

  expect_false(is_combined(
    graph = graph,
    klass_node(graph, "0113", "1838-01-01")
  ))

  ## Using `compare_node`

  # 0106 Fredrikstad

  expect_true(is_combined(
    graph = graph,
    node = klass_node(graph, "0106", "1994-01-01"),
    compare_node = klass_node(graph, "0103", "1964-01-01")
  ))

  expect_false(is_combined(
    graph = graph,
    node = klass_node(graph, "3004", "2020-01-01"),
    compare_node = klass_node(graph, "0106", "1994-01-01")
  ))

  # 1036 Fjotland

  expect_true(is_combined(
    graph = graph,
    node = klass_node(graph, "4227", "2020-01-01"),
    compare_node = klass_node(graph, "1036", "1840-01-01")
  ))
})

# Obtain the code and name for the desired levels of a classification.
# Input parameters: input level (level at which the classification is), desired levels, file with the classification as retrieved from KLASS.
#' Title
#'
#' @param input_level Classification level from the original dataset.
#' @param output_level Classification level for which the codes and names are desired by the user.
#' @param klass_data Classification file retrieved from KLASS.
#'
#' @return Data frame with the input and extra desired classification levels. Includes codes and names for each level.
#' @keywords internal
Levels <- function(input_level, output_level, klass_data) {
  # Create a variable for each level of the classification, starting from one variable which contains all levels.
  dt <- subset(klass_data, select = c("code", "parentCode", "level"))
  dt$level <- as.numeric(dt$level)
  klass_codes <- ConvertTable(dt)


  # Add codes and names in the same file (for now, called klass_codes).
  dt <- subset(klass_data, select = c("code", "name", "level"))
  max_level <- max(klass_data$level)
  dt <- subset(klass_data, select = c("code", "name"))
  for (temp_level in 1:max_level) {
    klass_codes <- merge(klass_codes, dt, by.x = c(paste("level", temp_level, sep = "")), by.y = c("code"))
    names(klass_codes)[length(names(klass_codes))] <- paste("name", temp_level, sep = "")
  }


  # Add all the desired levels to the output file.
  output_level[[length(output_level) + 1]] <- input_level # Add input level to list of desired levels, to get all output levels.

  # Check if all the requested levels are valid.
  for (i in output_level) {
    if (i > max(klass_data$level)) {
      stop("You are trying to retrieve a classification level that is more detailed than what is available. Please choose a valid classification level.")
    }
  }

  levelcode1 <- paste("level", output_level, sep = "")
  levelcode2 <- paste("name", output_level, sep = "")
  levelscode <- c(levelcode1, levelcode2)
  output_file <- klass_codes[, levelscode]
  output_file <- unique(output_file)


  return(output_file)
}

#' Check which level the input data/vector is
#'
#' The input level is calculated as the mode (most common) hierarchical
#' level in the input vector.
#'
#' @param x - vector of character
#' @param klass_data - - the right formatting to the classification levels
#' @return The hierarchical level of the input data is returned.
#' @keywords internal
levelCheck <- function(x, klass_data) {
  m <- match(x, klass_data$code)
  tab <- table(klass_data[m, ]$level)
  input_level <- as.numeric(names(tab)[which.max(tab)])

  if (is.na(input_level)) {
    stop("Cannot find an input level, please check the input vector")
  }
  return(input_level)
}

test_that("apply_klass returns correct names for numeric codes", {
  data(klassdata)
  kommune_names <- apply_klass(klassdata$kommune,
    classification = 131,
    date = "2015-01-01"
  )
  expect_equal(kommune_names[1], "Sandefjord")
})


test_that("apply_klass returns correct names for numeric codes", {
  data(klassdata)
  kommune_names <- apply_klass(klassdata$kommune,
    classification = 131,
    date = "2015-01-01"
  )
  expect_equal(kommune_names[1], "Sandefjord")
})


test_that("apply_klass returns correct names for character codes", {
  data(klassdata)
  sektor_names <- apply_klass(c("INNL", "UTL", "B_FIN"),
    classification = 39,
    date = "2019-01-01"
  )
  expect_equal(sektor_names[2], "Utlandet")
})


test_that("apply_klass returns correct language", {
  data(klassdata)
  occ <- apply_klass(
    klassdata$occupation,
    classification = 7,
    language = "en",
    date = "2015-01-01",
    format = F
  )
  expect_equal(occ[1], "Bartenders")
  occ <- apply_klass(
    klassdata$occupation,
    classification = 7,
    language = "nb",
    date = "2015-01-01",
    format = F
  )
  expect_equal(occ[4], "Elektronikkingeniører")
})


test_that("apply_klass returns correctly for character codes", {
  sektor <- c("INNL", "UTL")
  sektor_apply <- apply_klass(sektor, 39, date = "2020-01-01")
  expect_equal(sektor_apply[2], "Utlandet")
})


test_that("Check levelCheck for numerical codes", {
  occ <- get_klass(7, date = "2020-01-01")
  expect_equal(levelCheck("031", occ), 3)
  expect_equal(levelCheck(c("0210", "021"), occ), 3)
  sn <- get_klass(6, date = "2020-01-01")
  expect_equal(levelCheck("03.1", sn), 3)
  expect_equal(levelCheck("A", sn), 1) # Character code but added in here
})


test_that("Check levelCheck for character codes", {
  sek <- get_klass(39, date = "2020-01-01")
  expect_equal(levelCheck("INNL", sek), 1)
  expect_equal(levelCheck(c("INNL", "B_FIN", "B_FIN", NA), sek), 2)
})


test_that("apply_klass can return a variant classification", {
  dat <- c("000", "101", "102", "103")
  dat <- c("01.21", "01.46", "10.61")

  dat_new <- klassR::apply_klass(dat,
    classification = 6,
    variant = 1616,
    date = "2021-01-02"
  )
  expect_equal(dat_new[2], "Svinehold")

  dat_new <- klassR::apply_klass(
    dat,
    classification = 6,
    variant = 1616,
    output_level = 1,
    date = "2021-01-02"
  )
  expect_equal(dat_new[3], "Industri")
})


test_that(
  "apply_klass works for variant classification with Norwegian characters in the variant name",
  {
    dat <- c("05", "01")
    new <- apply_klass(
      dat,
      classification = 6,
      variant = 1616,
      output_level = 1,
      output = "name",
      date = "2020-01-01"
    )
    expect_equal(new[1], "Bergverksdrift og utvinning")
  }
)


test_that("An error is correctly returned in the case of a null vector", {
  expect_error(apply_klass(NULL, 131), "The input vector is empty")
})


test_that("apply_klass works for classifications with varying number of digits", {
  dat <- c("56", "580")
  new <- apply_klass(dat, classification = 270, date = "2024-01-01")
  expect_false(new[1] == new[2])
  expect_false(all(is.na(new)))
})


test_that("Nace classification with missing formatting", {
  # simple example with all missing formatting
  dat <- c("01460", "45112", "45111", "45310")
  expect_warning(
    new <- apply_klass(dat, classification = 6, date = "2024-01-01"),
    "Number missing .: 4"
  )
  expect_equal(new[1], "Svinehold")

  # Check mixture of formatting
  dat <- c("45112", "45.111")
  expect_warning(
    new <- apply_klass(dat, classification = 6, date = "2024-01-01"),
    "Number missing .: 1"
  )
  expect_equal(
    new[1],
    "Detaljhandel med biler og lette motorvogner, unntatt motorsykler"
  )

  # checking NAs
  dat <- c("45.112", "45.111", NA)
  expect_warning(
    new <- apply_klass(dat, classification = 6, date = "2024-01-01"),
    "Number of NA: 1"
  )
  expect_equal(
    new[1],
    "Detaljhandel med biler og lette motorvogner, unntatt motorsykler"
  )
  expect_true(is.na(new[3]))
})


test_that("Municipality classification with missing formatting", {
  # example with all missing leading 0
  dat <- c("301", "301")
  expect_warning(
    new <- apply_klass(dat, classification = 131, date = "2024-01-01"),
    "Number missing leading 0: 2"
  )
  expect_equal(new[1], "Oslo")

  # simple example with all missing leading 0
  dat <- c("301", "301")
  expect_warning(
    new <- apply_klass(dat, classification = 131, date = "2024-01-01"),
    "Number missing leading 0: 2"
  )
  expect_equal(new[1], "Oslo")


  # Check mixture of formatting
  dat <- c("45112", "45.111")
  expect_warning(
    new <- apply_klass(dat, classification = 6, date = "2024-01-01"),
    "Number missing .: 1"
  )
  expect_equal(
    new[1],
    "Detaljhandel med biler og lette motorvogner, unntatt motorsykler"
  )
})


test_that("apply_klass works for classifications with varying digits and letters", {
  dat <- c("56", "580", "KG1")
  new <- apply_klass(dat, classification = 270, date = "2024-01-01")
  expect_false(all(is.na(new)))

  dat <- c("01", "03b")
  new2 <- apply_klass(dat, classification = 207, date = "2024-01-01")
  expect_false(all(is.na(new2)))
})


test_that("An error is correctly returned in the case of a null vector", {
  expect_error(apply_klass(NULL, 131), "The input vector is empty")
})

#' Code level convert table (internal function)
#'
#' @param klass_data  Klass data frame to convert
#' @param code Name of variable containing code (Default = "code")
#' @param parentcode Name of variable of parent code (Default = "parentCode")
#' @param level Name of variable containing level (Default = "level")
#' @keywords internal
#' @return A dataframe
ConvertTable <- function(klass_data, code = "code", parentcode = "parentCode", level = "level") {
  levelset <- as.numeric(max(klass_data[, level]))
  uni <- unique(klass_data[klass_data[, level] == levelset, code])
  dx <- matrix(NA, length(uni), levelset)
  dx <- as.data.frame(dx)
  dx[, levelset] <- as.character(uni)
  for (i in 1:(levelset - 1)) {
    m <- match(dx[, levelset - i + 1], klass_data[, code])
    dx[, levelset - i] <- as.character(klass_data[, parentcode][m])
  }
  names(dx) <- paste("level", 1:levelset, sep = "")
  return(dx)
}

#' Conversion to character
#'
#' @param x a number or vector of numbers
#' @keywords internal
#' @return x converted to a string or vector of strings.
MakeChar <- function(x) {
  if (length(x) == 1) {
    xnew <- as.character(x)
  }
  if (length(x) > 1) {
    xnew <- sapply(x, FUN = MakeChar)
  }
  return(xnew)
}


#' Match and convert a classification
#'
#' @param x Input vector of classification codes. Vector must match "code" column from a call to get_klass().
#' @param classification Classification number
#' @param date String for the required date of the classification. Format must be "yyyy-mm-dd". For an inverval, provide two dates as a vector. If blank, will default to today's date.
#' @param variant The classification variant to fetch (if a variant is wanted).
#' @param correspond ID number for target in correspondence table. For correspondence between two dates within the same classification, use correspond = TRUE.
#' @param language Default "nb" for Norwegian (Bokmål). Also "nn" (Nynorsk) and "en" (English available for some classifications)
#' @param output_level Desired output level
#' @param output String describing output. May be "name" (default), "code" or "both".
#' @param format Logical for whther to run formatting av input vector x (Default = TRUE), important to check if formatting is in one level.
#'
#' @return A vector or data frame is returned with names and/or code of the desired output level.
#' @export
#'
#' @examples
#' data(klassdata)
#' kommune_names <- apply_klass(
#'   x = klassdata$kommune,
#'   classification = 131,
#'   language = "en",
#'   format = FALSE
#' )
apply_klass <- function(x,
                        classification,
                        date = NULL,
                        variant = NULL,
                        correspond = NULL,
                        language = "nb",
                        output_level = NULL,
                        output = "name",
                        format = TRUE) {
  # Check and standardise variables
  classification <- MakeChar(classification)
  if (is.null(x)) {
    stop("The input vector is empty.")
  }
  x <- MakeChar(x)

  if (is.null(date)) {
    date <- Sys.Date()
  }

  type <- ifelse(is.null(correspond), "vanlig", "kor")
  type <- ifelse(isTRUE(correspond), "change", type)
  type <- ifelse(is.null(variant), type, "variant")

  # Fetch classification table
  klass_data <- get_klass(
    classification,
    date = date,
    correspond = NULL,
    variant = variant,
    language = language,
    output_level = NULL
  )

  # Extract correspondence table
  if (type == "kor") {
    cor_table <- get_klass(
      classification,
      date = date,
      correspond = correspond,
      language = language
    ) # , output_level = output_level)

    new_table <- get_klass(
      classification = correspond,
      date = date,
      correspond = NULL,
      language = language
    ) # , output_level = output_level)
  }
  if (type == "change") {
    cor_table <- get_klass(
      classification = classification,
      date = date,
      correspond = TRUE,
      language = language,
      output_level = NULL
    )
  }

  # Formattering - only for nace and municipality
  if (format == TRUE & classification %in% c("6", "131")) {
    x_formatted <- formattering(x, classification = classification)
  } else {
    x_formatted <- x
  }

  # Check input data for level
  input_level <- levelCheck(x = x_formatted, klass_data = klass_data) # implies all are same level!
  if (is.null(output_level)) {
    output_level <- input_level
  }

  if (!all(input_level == output_level) &
    type %in% c("kor", "change")) {
    stop("Level changes and time changes/correspondence concurrently is not programmed.")
  }

  # Run level function
  if (!all(input_level == output_level) & is.null(correspond)) {
    x_level <- Levels(
      input_level = input_level,
      output_level = output_level,
      klass_data = klass_data
    )
  }

  if (all(input_level == output_level)) {
    x_level <- klass_data[klass_data$level == input_level, ]
    x_level[, paste("level", input_level, sep = "")] <- x_level$code
    x_level[, paste("name", input_level, sep = "")] <- x_level$name
  }


  # run matching
  levelcode <- paste("level", input_level, sep = "")
  if (type %in% c("vanlig", "variant")) {
    m <- match(x_formatted, x_level[, levelcode]) ### sjekk rekkefolge
  }

  if (type == "kor") {
    m1 <- match(x_level[, levelcode], cor_table[, "sourceCode"])
    m2 <- match(x_formatted, cor_table[, "sourceCode"])
    m3 <- match(cor_table[, "targetCode"], new_table[, "code"]) ## ?
  }

  if (type == "change") {
    m1 <- match(x_formatted, x_level[, levelcode])
    m2 <- match(x_formatted, cor_table$sourceCode)
  }

  # Choose format output
  if (type %in% c("vanlig", "variant")) {
    if (output == "code") {
      vars <- paste("level", output_level, sep = "")
    }
    if (output == "name") {
      vars <- paste("name", output_level, sep = "")
    }
    if (output == "both") {
      vars <- paste(c("level", "name"), output_level, sep = "")
    }
    out <- x_level[m, vars]
  }
  if (type == "kor") {
    if (output == "code") {
      vars <- "targetCode"
      vars2 <- "code"
    }
    if (output == "name") {
      vars <- "targetName"
      vars2 <- "name"
    }
    if (output == "both") {
      vars <- c("targetCode", "targetName")
      vars2 <- c("code", "name")
    }
    out <- cor_table[m2, vars]
  }
  if (type == "change") {
    if (output == "code") {
      vars <- "targetCode"
      vars2 <- "code"
    }
    if (output == "name") {
      vars <- "targetName"
      vars2 <- "name"
    }
    if (output == "both") {
      vars <- c("targetCode", "targetName")
      vars2 <- c("code", "name")
    }
    out <- ifelse(!is.na(m2), cor_table[m2, vars], klass_data[m1, vars2])
  }
  return(out)
}


#' @rdname apply_klass
#' @param klass Deprecated; use `classification` instead.
#' @export
ApplyKlass <- function(x,
                       klass,
                       date = NULL,
                       variant = NULL,
                       correspond = NULL,
                       language = "nb",
                       output_level = NULL,
                       output = "name",
                       format = TRUE) {
  # .Deprecated("apply_klass") # add in for future versions
  apply_klass(
    x = x,
    classification = klass,
    date = date,
    variant = variant,
    correspond = correspond,
    language = language,
    output_level = output_level,
    output = output,
    format = format
  )
}

## to do:
# level input need and run level check first but the formatted input is need to check level....FIX

#' Format vector for industry codes
#' @param x - vector of character
#' @param classification - classification number
#' @return vector of character
#' @keywords internal
formattering <- function(x, classification) {
  # Check for missing values
  miss <- sum(is.na(x) | x == "")
  if (miss != 0) {
    warning(c("Number of NA: ", miss))
  }

  # Check and format
  if (classification == 6) {
    x_formatted <- formattering_nace(x)
  }
  if (classification == 131) {
    x_formatted <- formattering_kommune(x)
  }

  x_formatted
}


#' Format vector for industry codes
#' @param x Character vector
#' @return Formatted charcter vector
#' @keywords internal
formattering_nace <- function(x) {
  # Specify position of the .
  dot <- 3

  # Identify number missing punctuation
  x_nopunc <- tm::removePunctuation(x)
  x_over2 <- nchar(x_nopunc) >= 3
  x_over2[is.na(x_over2)] <- FALSE
  mangler_dot <- sum(!grepl("\\.", x[x_over2]))

  if (mangler_dot != 0) {
    warning(c("Number missing .: ", mangler_dot))
  }

  mangler_dot <- length(x) - sum(grepl("\\.", x))

  # Format
  x_formatted <- sub(paste0("(?<=.{", dot - 1, "})"), ".", x_nopunc, perl = TRUE)
  x_formatted[!x_over2] <- x_nopunc[!x_over2]

  x_formatted
}

#' Format vector for kommune codes
#' @param x - vector of character type for kommune codes
#' @return vector of character
#' @keywords internal
formattering_kommune <- function(x) {
  # Check for letters
  if (any(grepl("^[A-Za-z]+$", x))) {
    stop("Letters detected in kommune codes. Please check and rerun.")
  }

  # Find dropped leading zeros
  mangler0 <- sum(nchar(x) < 4, na.rm = T)

  # Fix if any
  if (mangler0 > 0) {
    x_formatted <- formatC(as.numeric(x), width = 4, flag = "0")
    warning(c("Number missing leading 0: ", mangler0))
  } else {
    x_formatted <- x
  }
  x_formatted
}

test_that("Formatting works for nace codes of differnt levels", {
  x <- c("01", "011", "0110", "01110")
  suppressWarnings(codes <- klassR:::formattering(x, classification = 6))
  expect_equal(codes, c("01", "01.1", "01.10", "01.110"))
})

test_that("Formatting works for nace codes with letters", {
  x <- c("A", "01", "011", "0110", "01110")
  suppressWarnings(codes <- klassR:::formattering(x, classification = 6))
  expect_equal(codes, c("A", "01", "01.1", "01.10", "01.110"))
})

test_that("Formatting works for kommune", {
  x <- c("301", "0301", "1101", "1103")
  suppressWarnings(codes <- klassR:::formattering(x, classification = 131))
  expect_equal(codes, c("0301", "0301", "1101", "1103"))
})

test_that("Kommune fails when letters present", {
  x <- c("Oslo", "0301", "1101", "1103")
  expect_error(codes <- klassR:::formattering(x, classification = 131))
})

#' Internal function to check date
#' @param date Date
#' @keywords internal
#' @return NULL
CheckDate <- function(date) {
  dcheck <- try(as.Date(date, format = "%Y-%m-%d"))
  if (!inherits(dcheck, "Date") | is.na(dcheck)) {
    stop("An incorrect date format was given. Please use format 'YYYY-mm-dd'.")
  }
  if (dcheck < "1500-01-01") {
    stop("An incorrect date format was given. Please use format 'YYYY-mm-dd'.")
  }
}


#' Internal function to create URL address
#'
#' @param classification Classification number
#' @param correspond Target number for correspondence table
#' @param variant_name The name of the variant of the classification
#' @param type String describing type. "vanlig" for normal classification and "kor" for correspondence. Default = "vanlig"
#' @param fratil True/False for whether a date interval is to be used. Default = False
#' @param date Date(s) for classification
#' @param output_level_coding Coding for output level
#' @param language_coding Coding for language
#' @keywords internal
#'
#' @return String url adress
MakeUrl <- function(
  classification,
  correspond = NULL,
  correspondID = NULL,
  variant_name = NULL,
  type = "vanlig",
  fratil = FALSE,
  date = NULL,
  output_level_coding = NULL,
  language_coding = NULL
) {
  # Standard classification/codelist
  if (type == "vanlig" & fratil == TRUE) {
    coding <- paste0("/codes?from=", date[1], "&to=", date[2])
  }
  if (type == "vanlig" & fratil == FALSE) {
    coding <- paste0("/codesAt?date=", date)
  }

  # For correspondence tables
  if (type == "kor" & fratil == TRUE) {
    coding <- paste(
      "/corresponds?targetClassificationId=",
      MakeChar(correspond),
      "&from=",
      date[1],
      "&to=",
      date[2],
      sep = ""
    )
  }
  if (type == "kor" & fratil == FALSE) {
    coding <- paste(
      "/correspondsAt?targetClassificationId=",
      MakeChar(correspond),
      "&date=",
      date,
      sep = ""
    )
  }
  if (type == "korID") {
    coding <- paste0("correspondencetables/", MakeChar(correspondID))
    # Tables with given ID can not be combined with other parameters in the call
    classification <- ""
    language_coding <- ""
    output_level_coding <- ""
  }

  # For time changes
  if (type == "change") {
    coding <- paste0("/changes?from=", date[1], "&to=", date[2])
  }

  # For fetching a variant
  if (type == "variant" & fratil == TRUE) {
    coding <- paste0(
      "/variant?variantName=",
      variant_name,
      "&from=",
      date[1],
      "&to=",
      date[2]
    )
  }
  if (type == "variant" & fratil == FALSE) {
    coding <- paste0("/variantAt?variantName=", variant_name, "&date=", date)
  }

  # For future times
  idag <- Sys.Date()
  if ((idag < date[1]) & (type != "korID")) {
    message(
      "The date you selected is in the future. You may be viewing a future classification that is not currently valid"
    )
    coding <- paste0(coding, "&includeFuture=True")
  } else if ((length(date) > 1) & (type != "korID")) {
    if (idag < date[2]) {
      message(
        "The date you selected is in the future. You may be viewing a future classification that is not currently valid"
      )
      coding <- paste0(coding, "&includeFuture=True")
    }
  }

  # Whether classifications is needed in url
  if (type == "korID") {
    classifics <- ""
  } else {
    classifics <- "classifications/"
  }

  # Paste together to an URL
  url <- paste(
    GetBaseUrl(),
    classifics,
    classification,
    coding,
    output_level_coding,
    language_coding,
    sep = ""
  )
  return(url)
}


#' Check connection
#' Function to check that a connection to data.ssb.no is able to be established
#' @param url String url address for connection to check
#' @keywords internal
#' @return Nothing is returned but a error or warning message is return if no connection is available
check_connect <- function(url) {
  tryget <- tryCatch(
    httr::GET(url = utils::URLencode(url)),
    error = function(e) conditionMessage(e),
    warning = function(w) conditionMessage(w)
  )
  if (!inherits(tryget, "response")) {
    message(tryget)
    return(invisible(NULL))
  } else if (httr::http_error(tryget$status_code)) {
    if (httr::http_type(tryget) == "application/json") {
      response <- jsonlite::fromJSON(rawToChar(tryget$content))

      if ("detail" %in% names(response)) {
        message(
          paste("Connection failed with error code", tryget$status_code),
          ":\n",
          response$detail
        )
      }
    } else {
      message(paste("Connection failed with error code", tryget$status_code))
    }

    return(invisible(NULL))
  }
  tryget
}


#' Stop quietly function
#' Stop from a function without an error. Used for stopping when no internet
#' @keywords internal
stop_quietly <- function() {
  opt <- options(show.error.messages = FALSE)
  on.exit(options(opt))
  stop()
}


#' Get variant name
#' Internal function for fetching the variant name based on the number
#' @param variant The variant number
#' @keywords internal
get_variant_name <- function(variant) {
  # Check variant url and that it exists
  url <- paste0(GetBaseUrl(), "variants/", variant)
  variant_url <- check_connect(url)
  if (is.null(variant_url)) {
    stop_quietly()
  }

  # Extract text with variant name
  variant_text <- httr::content(variant_url, "text", encoding = "UTF-8") ####
  if (grepl("variant not found", variant_text)) {
    stop("The variant ", variant, " was not found.")
  }
  variant_name_full <- jsonlite::fromJSON(variant_text, flatten = TRUE)$name
  # Earlier have just taken first word before numbers but this is not working so use whole name
  # name_sm <- strsplit(variant_name_full, split = "(?<=[a-zA-Z])\\s*(?=[0-9])", perl = T)[[1]][1]

  # Substitute for spaces, and norwegian characters
  variant_name_full <- gsub(" ", "%20", variant_name_full)
  variant_name_full <- gsub("\u00E6", "%C3%A6", variant_name_full)
  variant_name_full <- gsub("\u00F8", "%C3%B8", variant_name_full)
  variant_name_full <- gsub("\u00E5", "%C3%A5", variant_name_full)
  variant_name_full <- gsub("\\(", "%28", variant_name_full)
  variant_name_full <- gsub("\\)", "%29", variant_name_full)

  variant_name_full
}


#' Get json file from Url - alternative version
#'
#' @param url String url address
#' @param check Logical parameter on whether to check if the url exists
#' @keywords internal
#' @return text in json format
GetUrl2 <- function(url, check = TRUE) {
  # Fetch contents from classfication
  if (check) {
    hent_klass <- check_connect(url)
  } else {
    hent_klass <- httr::GET(url = url)
  }
  if (is.null(hent_klass)) {
    return(invisible(NULL))
  }
  klass_text <- httr::content(hent_klass, "text", encoding = "UTF-8") ## deserialisering with httr function
  return(klass_text)
}

#' Fetch Statistics Norway classification data using API
#'
#' @param classification Number/string of the classification ID/number. (use klass_list() to find this)
#' @param date String for the required date of the classification. Format must be "yyyy-mm-dd". For an inverval, provide two dates as a vector. If blank, will default to today's date.
#' @param correspond Number/string of the target classification for correspondence table (if a correspondence table is requested).
#' @param correspondID ID number of the correspondence table to retrieve. Use as an alternative to correspond.
#' @param variant The classification variant to fetch (if a variant is wanted).
#' @param output_level Number/string specifying the requested hierarchy level (optional).
#' @param language Two letter string for the requested language output. Default is Bokmål ("nb"). Nynorsk ("nn") and English ("en") also available for some classification.)
#' @param output_style String variable for the output type. Default is "normal". Specify "wide" for a wide formatted table output.
#' @param notes Logical for if notes should be returned as a column. Default FALSE
#' @param quiet Logical for whether to suppress the printing of the API address. Default TRUE.
#'
#' @return The function returns a data frame of the specified classification/correspondence table. Output variables include:
#' code, parentCode, level, and name for standard lists. For correspondence tables variables include:
#' sourceCode, sourceName, targetCode and targetName. For date correspondence tables variables include:
#' oldCode, oldName, newCode and newName. For "wide" output, code and name with level suffixes is specified.
#' For date ranges, validFromInRequestedRange and validToInRequestedRange give the dates for the classification. Variable ChangeOccured gives the
#' effective date for classification change in classification change tables.
#' @export
#'
#' @examples
#' # Get classification for occupation classifications
#' head(get_klass(classification = "7"))
#' # Get classification for occupation classifications in English
#' head(get_klass(classification = "7", language = "en"))
get_klass <- function(
  classification,
  date = NULL,
  correspond = NULL,
  correspondID = NULL,
  variant = NULL,
  output_level = NULL,
  language = "nb",
  output_style = "normal",
  notes = FALSE,
  quiet = TRUE
) {
  # create type of classification for using later
  type <- ifelse(is.null(correspond) & is.null(correspondID), "vanlig", "kor")
  type <- ifelse(isTRUE(correspond), "change", type)
  type <- ifelse(is.null(correspond) & type == "kor", "korID", type)
  type <- ifelse(is.null(variant), type, "variant")

  # check classification is a character
  if (is.null(correspondID)) {
    classification <- MakeChar(classification)
  } else {
    classification <- ""
  }

  # dato sjekking
  if (!is.null(date[1]) & (!is.null(correspondID))) {
    message(
      "Note: Correspondence tables provided using an ID do not har a date attached. Date is being ignored."
    )
  }
  if (is.null(date[1])) {
    date <- Sys.Date()
  }

  # Create variables fratil (whether to and from dates should be used) and ver
  if (length(date) == 1 & !is.numeric(date)) {
    fratil <- FALSE # om det er dato fra og til
    ver <- FALSE # om det skal finne en versjon (not implemented yet)
    CheckDate(date)
  }

  if (length(date) == 1 & is.numeric(date)) {
    fratil <- FALSE
    ver <- TRUE
  }

  # if two dates are provided, check both and check order. Swap if neccessary
  if (length(date) == 2) {
    fratil <- TRUE
    ver <- FALSE
    CheckDate(date[1])
    CheckDate(date[2])
    if (difftime(as.Date(date[2]), as.Date(date[1])) < 0) {
      date <- date[c(2, 1)]
      dateswap <- TRUE
    } else {
      dateswap <- FALSE
    }
    date[2] <- as.character(as.Date(date[2], format = "%Y-%m-%d") + 1)
  }
  if (length(date) > 2) {
    stop("You have provided too many dates.")
  }

  # Check levels
  if (is.null(output_level)) {
    output_level_coding <- ""
  } else {
    output_level_coding <- paste("&selectLevel=", output_level, sep = "")
  }

  # Set language coding
  language_coding <- paste("&language=", language, sep = "")

  # Create url and collect data
  if (type == "variant") {
    if (is.numeric(variant) | grepl("^[0-9]", variant)) {
      variant_name <- get_variant_name(variant)
    } else {
      variant_name <- gsub(" ", "%20", variant)
    }
    if (!is.null(output_level)) {
      print(
        "Selecting an output level for a variant isn't currently supported. All levels will be returned"
      )
    }
  }
  url <- MakeUrl(
    classification = classification,
    correspond = correspond,
    correspondID = correspondID,
    variant_name = variant_name,
    type = type,
    fratil = fratil,
    date = date,
    output_level_coding = output_level_coding,
    language_coding = language_coding
  )
  if (!quiet) {
    print(paste("Fetching class from:", url))
  }
  if (type == "kor") {
    klass_text <- GetUrl2(url)

    if (is.null(klass_text)) {
      stop_quietly()
    }

    if (grepl("no correspondence table", klass_text)) {
      stop(
        "No correspondence table found between classes ",
        classification,
        " and ",
        correspond,
        " for the date ",
        date,
        "For a list of valid correspondence tables use the function correspond_list()"
      )
    }
  } else {
    klass_text <- GetUrl2(url)
  }
  if (is.null(klass_text)) {
    stop_quietly()
  }

  if (klass_text == '{\"codes\":[]}') {
    stop(
      "No codes were found for classification ",
      classification,
      " with the current search parameters.",
      " The specified date (",
      date,
      ") may be too early."
    )
  }

  if (grepl("not found", klass_text)) {
    stop(
      "No classification table was found for classification number ",
      classification,
      ".
    Please try again with a different classification number.
    For a list of possible classification's use the function list_klass() or list_family()"
    )
  }
  if (grepl("not published in language", klass_text)) {
    stop(
      "The classification requested was not found for language = ",
      gsub(".*=", "", language_coding)
    )
  }
  if (grepl("does not have a variant named", klass_text)) {
    stop(
      "The variant ",
      variant,
      " was not found for classification number ",
      classification
    )
  }

  if (type %in% c("vanlig", "variant")) {
    klass_data <- jsonlite::fromJSON(klass_text, flatten = TRUE)$codes
    klass_data <- klass_data[, c("code", "parentCode", "level", "name")]
  }
  if (type == "kor") {
    klass_data <- jsonlite::fromJSON(
      klass_text,
      flatten = TRUE
    )$correspondenceItems
    if (length(klass_data) == 0) {
      stop(
        "No correspondence table found between classes ",
        classification,
        " and ",
        correspond,
        " for the date ",
        date,
        "For a list of valid correspondence tables use the function correspond_list()"
      )
    }

    klass_data <- klass_data[, c(
      "sourceCode",
      "sourceName",
      "targetCode",
      "targetName"
    )]
  }
  if (type == "korID") {
    klass_data <- jsonlite::fromJSON(
      klass_text,
      flatten = TRUE
    )$correspondenceMaps
  }
  if (type == "change") {
    klass_data <- jsonlite::fromJSON(klass_text, flatten = TRUE)$codeChanges
    if (!is.data.frame(klass_data)) {
      stop("No changes found for this classification.")
    }
    if (dateswap) {
      klass_data <- klass_data[, c(
        "newCode",
        "newName",
        "oldCode",
        "oldName",
        "changeOccurred"
      )]
    } else {
      klass_data <- klass_data[, c(
        "oldCode",
        "oldName",
        "newCode",
        "newName",
        "changeOccurred"
      )]
    }
    names(klass_data) <- c(
      "sourceCode",
      "sourceName",
      "targetCode",
      "targetName",
      "changeOccurred"
    )
  }
  if (type %in% c("variant", "vanlig") & isTRUE(notes)) {
    klass_data$notes <- jsonlite::fromJSON(
      klass_text,
      flatten = TRUE
    )$codes$notes
  }

  if (type %in% c("variant", "vanlig") & isTRUE(fratil)) {
    klass_data$validFromInRequestedRange <- jsonlite::fromJSON(
      klass_text,
      flatten = TRUE
    )$codes$validFromInRequestedRange
    klass_data$validToInRequestedRange <- jsonlite::fromJSON(
      klass_text,
      flatten = TRUE
    )$codes$validToInRequestedRange
  }

  if (
    output_style == "wide" &
      is.null(output_level) &
      is.null(correspond) &
      is.null(correspondID)
  ) {
    # get maximum level
    maxlength <- max(klass_data$level)
    minlength <- min(klass_data$level)

    # check several levels exist
    if (maxlength == minlength) {
      warning(
        "Only one level was detected. Classification returned with output_style normal. "
      )
      return(as.data.frame(klass_data))
    }

    # create most detailed level dataframe
    wide_data <- klass_data[klass_data$level == maxlength, c("code", "name")]
    names(wide_data) <- c(paste0("code", maxlength), paste0("name", maxlength))

    # loop through and match the other levels
    for (i in (as.numeric(maxlength) - 1):minlength) {
      m1 <- match(wide_data[, paste0("code", (i + 1))], klass_data$code)
      m2 <- match(klass_data$parentCode[m1], klass_data$code)
      tmp <- data.frame(klass_data$parentCode[m1], klass_data$name[m2])
      names(tmp) <- c(paste0("code", i), paste0("name", i))

      # paste in wide format beside previous
      wide_data <- cbind(wide_data, tmp)
    }

    # rename as klass_data for return
    klass_data <- wide_data
  }

  as.data.frame(klass_data)
}


#' @rdname get_klass
#' @param klass Deprecated; use `classification` instead.
#' @export
GetKlass <- function(
  klass,
  date = NULL,
  correspond = NULL,
  correspondID = NULL,
  variant = NULL,
  output_level = NULL,
  language = "nb",
  output_style = "normal",
  notes = FALSE,
  quiet = TRUE
) {
  # .Deprecated("get_klass") # Add in for future versions
  get_klass(
    classification = klass,
    date = date,
    correspond = correspond,
    correspondID = correspondID,
    variant = variant,
    output_level = output_level,
    language = language,
    output_style = output_style,
    notes = notes,
    quiet = quiet
  )
}

# Code for tests for the function get_klass

test_that("get_klass returns a classification", {
  class_data <- get_klass(131, date = "2015-01-01")
  expect_equal(class_data$code[1], "0101")
})


test_that("get_klass returns a classification with Norwegian letters", {
  class_data <- get_klass(131, date = "2015-01-01")
  expect_equal(class_data$name[8], "Rømskog")
})


test_that("get_klass returns no error when no date given", {
  expect_type(get_klass(131), type = "list")
})


test_that("get_klass returns error when date in wrong format", {
  expect_error(
    x <- get_klass(2, date = "01-01-2024"),
    "An incorrect date format was given. Please use format 'YYYY-mm-dd'."
  )

  expect_error(
    x <- get_klass(2, date = "20240101"),
    "An incorrect date format was given. Please use format 'YYYY-mm-dd'."
  )

  expect_error(
    x <- get_klass(
      2,
      date = c("2024-01-01", "2023-01-01", "2022-01-01"),
      "You have provided too many dates."
    )
  )
})


test_that("get_klass returns changes when two dates given", {
  class_data1 <- get_klass(131, date = c("2022-01-01", "2023-01-01"))
  class_data2 <- get_klass(131, date = c("2023-01-01", "2022-01-01"))
  expect_equal(nrow(class_data1), nrow(class_data2))
})


test_that("get_klass returns classifications with future date", {
  expect_message(
    class_data <- get_klass(2, date = "2100-01-01"),
    "The date you selected is in the future. You may be viewing a future classification that is not currently valid"
  )
  expect_equal(nrow(class_data), 2)
})


test_that("get_klass returns correct output level", {
  class_data <- get_klass(7, date = "2024-01-01", output_level = 1)
  expect_equal(nrow(class_data), 10)
  expect_equal(class_data$name[2], "Ledere")
})


test_that("get_klass returns english language", {
  class_data <- get_klass(2, date = "2024-01-01", language = "en")
  expect_equal(class_data$name, c("Male", "Female"))
})


test_that("get_klass returns a correspondence table", {
  class_data <- get_klass(104, correspond = 131, date = "2015-01-01")
  expect_equal(class_data$sourceCode[1], "01")
})


test_that("get_klass returns a correspondence table in both directions", {
  class_data1 <- get_klass(104, correspond = 131, date = "2020-02-01")
  class_data2 <- get_klass(131, correspond = 104, date = "2020-02-01")
  expect_equal(class_data1$sourceCode[1], "03")
  expect_equal(class_data2$sourceCode[1], "0301")
})


test_that("get_klass returns a valid variant including ones with spaces in name", {
  variant_data <- get_klass(6, variant = 1616, date = "2021-01-02")
  expect_equal(variant_data$name[2], "Jordbruk, skogbruk og fiske")

  variant_data <- get_klass(6, variant = 1615, date = "2021-01-02")
  expect_equal(variant_data$name[6], "Dyrking av ris")
})

test_that("get_klass returns a correspondence table", {
  cor_data1 <- get_klass(131, correspond = 127, date = "2024-01-02")
  cor_data2 <- get_klass(127, correspond = 131, date = "2024-01-02")

  expect_gt(nrow(cor_data1), 1)

  expect_equal(cor_data1$sourceCode, cor_data2$targetCode)
})

test_that("get_klass returns a correspondent using ID", {
  expect_message(
    cor_data <- get_klass(correspondID = 1441, date = "2024-01-02")
  )
  expect_equal(cor_data$targetCode[1], "3100")
})


test_that("get_klass fails quietly with invalid variant", {
  # Capture the warnings and errors
  err <- NULL
  result <- tryCatch(
    {
      variant_data <- get_klass(6, variant = 1, date = "2021-01-02")
    },
    error = function(e) {
      err <<- e
      NULL
    }
  )

  # Check that an error was indeed thrown
  expect_true(!is.null(err))

  # Check that the error message is empty
  expect_true(nchar(conditionMessage(err)) == 0)
})


test_that("get_klass returns notes when requested", {
  class_data <- get_klass(277, notes = TRUE, date = "2023-01-12")
  note <- class_data$notes[class_data$code == "FGK8"]
  expect_equal(
    note,
    "Grupperingen omfatter funksjonene 202, 215, 222 og 223 innen grunnskole."
  )
})


test_that("get_klass returns a future classification", {
  expect_message(classdata <- get_klass(104, date = "2050-01-01"))
  expect_equal(classdata$name[6], "Akershus")
})


test_that("get_klass returns a correspondence table using ID", {
  expect_message(
    classdata <- get_klass(correspondID = 1111, date = "2023-01-12")
  )
  expect_equal(classdata$targetName[1], "Oslo")
})

test_that("get_klass returns valid dates for a date range", {
  class_data <- get_klass(131, date = c("2020-01-01", "2024-01-02"))
  expect_equal(class_data$validFromInRequestedRange[1], "2020-01-01")

  variant_data <- get_klass(
    6,
    variant = 1616,
    date = c("2020-01-01", "2024-01-02")
  )
  expect_equal(variant_data$validFromInRequestedRange[1], "2020-01-01")
})

test_that("get_klass gives an informative error message when a correspondence is missing", {
  expect_snapshot(
    get_klass(131, correspond = 556, date = "2020-01-01"),
    error = TRUE
  )
})

test_that("get_klass gives an informative error message when a date is too old", {
  expect_snapshot(
    get_klass(131, date = "1600-01-01"),
    error = TRUE
  )
})

#' Testdata for klassR package
#'
#' A dataset containing variables for testing of Statistics Norways classification API with the klassR package. Some observations are missing or incorrect for testing and demonstrations.
#'
#' @format A data frame containing 100 rows and 7 variables:
#' \describe{
#' \item{ID}{Identification number}
#' \item{sex}{1/2 variable for sex}
#' \item{education}{4-digit number for education standard ISCED97 (level and subject area) NUS (klass = 66) 2015.01.01}
#' \item{kommune}{4-digit code for Norwegian municipality (klass = 131). Based on 2015.01.01}
#' \item{kommune2}{Numeric variable for Norwegian municipality with dropped leading zero's for testing (klass = 131). Based on 2015.01.01}
#' \item{nace5}{5-digit code for industry (NACE). Based on 01.01.2015 standard industry codes (klass = 7)}
#' \item{occupation}{4-digit occupation codes using standard for STYRK-08 (klass = 7) 2015.01.01}
#' }
"klassdata"


#' Test Graph data for municipalities in 2024
#'
#' A nested list of graph data for using in testing
#'
"klass_131_graph"


#' Test Graph data for municipalities in 2020
#'
#' A nested list of graph data for using in testing
#'
"klass_131_2020_graph"


#' Test Graph data for municipalities in 1964
#'
#' A nested list of graph data for using in testing
#'
"klass_131_1964_graph"

#' Test data for changes in municipality codes from 1963 onwards
#'
#' A data.frame containing information about changes in municipality codes
#'
"api_endringer_1963"

#' Test data for changes in municipality codes from 2019 onwards
#'
#' A data.frame containing information about changes in municipality codes
#'
"api_endringer_2019"

#' Find the equivalent sets of a node at various dates
#'
#' @param node The node that we're finding the equivalent sets of
#' @param dates The dates that we want to find the equivalent sets in
#' @param graph The graph that the nodes come from
#'
#' @return A named list of `length(dates)` containing the equivalent nodes for
#'   each date. The names of the returned list are the dates provided in `date`,
#'   coerced to character.
#'
find_equivalent_nodes <- function(node, dates, graph) {
  related_nodes <- igraph::bfs(
    graph = graph,
    root = node,
    mode = "all",
    unreachable = FALSE
  )[["order"]]

  result <- lapply(dates, function(date) {
    result_nodes <- related_nodes[
      date >= related_nodes$validFrom &
        (date < related_nodes$validTo | is.na(related_nodes$validTo))
    ]

    result_nodes[order(result_nodes$code)]
  })

  names(result) <- as.character(dates)

  return(result)
}

#' Find equivalent sets of codes in a Klass classification
#'
#' Find which codes should be combined to reconstruct older or newer codes
#' across multiple dates. Creates groups dynamically to fit the provided date
#' range, with flexible labelling functionality.
#'
#'
#' @param classification The Klass classification to be used
#' @param dates The dates that equivalent sets of codes should be found for.
#' @param labels `TRUE`, `FALSE` or a named list of functions.
#'
#'   This parameter controls whether or not to add group labels to groups of
#'   equivalent sets. If `TRUE`, labels are constructed using the codes valid at
#'   the latest provided date, and comma separated like in the example below. By
#'   default, labels will be placed in a column named `label`.
#'
#'   ```
#'   "1508 Ålesund, 1580 Haram"
#'   ```
#'   If `FALSE`, no labels will be applied.
#'
#'   This parameter also accepts a named list of labelling functions. The
#'   resulting dataset will contain a label column for each of the supplied
#'   functions. The names of the label columns are specified using the names of
#'   the list of functions.
#'
#'   The functions provided in this parameter can accept any of the following
#'   parameters: `date`, `code`, `name`, `validFrom` and `validTo`, representing
#'   the corresponding values of each code in a group. The functions must also
#'   provide a `...` parameter, unless using all of the above. The functions can
#'   expect that the input variables have the same length of 1 or longer. The
#'   functions should return a character vector of length one or the same length
#'   as the input variables.
#'
#'   The following list, when supplied to this parameter, creates two label
#'   columns: one containing the codes and names (`label1`), and another with
#'   only the codes `label2`. In this example, `label1` creates the same labels
#'   as the default labelling used when `labels == TRUE`.
#'
#'   ```
#'   list(
#'     label1 = function(code, name, date, ...) {
#'       label_codes <- date == max(date)
#'       paste(code[label_codes], name[label_codes], collapse = ", ")
#'     },
#'     label2 = function(code, date, ...) {
#'       label_codes <- date == max(date)
#'       paste(code[label_codes], collapse = ", ")
#'     }
#'   )
#'   ```
#'
#' @param graph Optional. Generating the graph using `klass_graph` manually
#'   beforehand and providing it in this parameter can save time if running
#'   `find_equivalent_codes` multiple times in sequence.
#'
#' @param date_format Optional. Passed directly to \link{format}, this is used to
#'   specify the output format for the `date` column. The default keeps just the
#'   year (`"YYYY"`). To get the full date in `"YYYY-MM-DD"` format, use
#'   `"%Y-%m-%d"`. See \link{strptime} for complete functionality.
#'
#' @return
#'   A data.frame with columns:
#'
#'   - `date` containing the input `dates`
#'   - `code` containing the set of equivalent codes in each date
#'   - `name` containing the names of each code
#'   - `validFrom` and `validTo` values for each code returned
#'   - By default, `labels`, giving a unique group label for each group of equivalent sets.
#'
#' @details This function provides a solution to the problem of split or
#'   combined codes in Klass classifications. When using \code{\link{update_klass}} to ask
#'   "what is this code in this version of the classification in this other
#'   version of the classification?", the answer is sometimes that the code has
#'   been split into two or more codes (or combined from two or more codes, if
#'   trying to back-date a code), and therefore that the code cannot be updated.
#'
#'   The solution provided by `find_equivalent_codes` is answering the question: "in
#'   these versions of the classification, which codes were equivalent to this
#'   code in this other version of the classification?".
#'
#'   Consider the following example of two codes combining into one. Here, `"a"`
#'   and `"b"` are valid at t1, and are combined into `"c"` at t2.
#'
#'   ```
#'   t1      t2
#'   a ---|
#'   b ---+-> c
#'
#'   ```
#'
#'   \code{\link{update_klass}} would inform us that `"a"` can be updated to
#'   `"c"` at t2, unless we specified `combine = FALSE`, in which case the
#'   result would be `NA`. `find_equivalent_codes()` would inform us that the
#'   equivalent of the codes `"a"` and `"b"` in t1 at t2 is `"c"`.
#'
#'   We can also consider a code splitting into two. In this example, `"a"` is
#'   valid at t1, and splits into `"b"` and `"c"` at t2.
#'
#'   ```
#'   t1     t2
#'   a
#'   |-----> b
#'   |-----> c
#'   ```
#'
#'   \code{\link{update_klass}} is unable to provide an updated code due to the
#'   split, and would return `NA`. `find_equivalent_codes` would inform us that the
#'   equivalent codes of `"a"` at t1 is `"b"` and `"c"` at t2.
#'
#'
#'   `find_equivalent_codes` can handle more than two dates. In the following
#'   example, `"a"` splits into `"b"` and `"c"` at t2, and `"b"` and `"c"`
#'   combine into `"d"` at t3. `find_equivalent_codes` can inform us that `"a"` is
#'   equivalent to `"b"` and `"c"` at t2, and `"d"` at t3.
#'
#'   ```
#'   t1     t2     t3
#'   a
#'   |-----> b |
#'   |-----> c +-> d
#'   ```
#'
#'   `find_equivalent_codes` will only search in the time range we specify. As a
#'   consequence, generating sets of equivalent codes over longer time spans
#'   will generally create larger sets than using shorter time spans.
#'
#'   To illustrate this behavior, we can add a new code `"e"` to the previous
#'   example, and have `"d"` and `"e"` combine into `"f"` at t4.
#'
#'   ```
#'   t1     t2     t3     t4
#'   a
#'   |-----> b |
#'   |-----> c |-> d |
#'   e---------------+--> f
#'   ```
#'
#'   Finding the
#'   equivalents of `"a"` in t1 at t2 and t3 returns the same sets as
#'   before:
#'
#'   - t1: `"a"`
#'   - t2: `"b"` and `"c"`
#'   - t3: `"d"`
#'
#'   However, if we also wanted to know the equivalent set for t4, the result
#'   would be:
#'
#'   - t1: `"a"` and `"e"`
#'   - t2: `"b"`, `"c"` and `"e"`
#'   - t3: `"d"` and `"e"`
#'   - t4: `"f"`
#'
#' `find_equivalents` is a legacy alias for `find_equivalent_codes`.
#'
#' @export
find_equivalent_codes <- function(
  classification,
  dates,
  labels = TRUE,
  graph = klass_graph(classification),
  date_format = "%Y"
) {
  # check if any provided dates are NA
  if (any(is.na(dates))) {
    stop("`dates` cannot be NA.")
  }

  # prepare dates lacking month and date information for date conversion
  if (any(nchar(dates) == 4)) {
    dates[nchar(dates) == 4] <- paste0(dates[nchar(dates) == 4], "-01-01")
  }

  dates <- as.Date(dates)

  if (any(is.na(dates))) {
    stop("Some dates could not be converted to date format.")
  }

  if (!length(dates) > 1) {
    stop("Need to provide at least two dates")
  }

  # Search in codes that were valid before or on max_date and valid to
  # after the min_date (or still valid)
  sgraph <- igraph::subgraph(
    graph,
    igraph::V(graph)$validFrom <= max(dates) &
      (is.na(igraph::V(graph)$validTo) | igraph::V(graph)$validTo >= min(dates))
  )

  result <- data.frame()

  for (i in 1:length(sgraph)) {
    node <- igraph::V(sgraph)[i]

    # build list of equivalent sets of nodes for this node
    equivalent_sets <- find_equivalent_nodes(node, dates, sgraph)

    # instantiate data.frame that will contain information about all equivalent
    # sets for this code
    equivalents_df <- data.frame()

    for (j in seq_along(equivalent_sets)) {
      # build data frame containing information about the codes in this set
      set_df <- as.data.frame(
        igraph::vertex.attributes(sgraph, equivalent_sets[[j]])
      )

      # add date variable specifying at what date this set is valid
      set_df$date <- format(dates[j], date_format)

      # select variables
      set_df <- set_df[, c("date", "code", "label", "validFrom", "validTo")]

      # rename `label` to `name` to match output from other klassR functions
      names(set_df)[3] <- "name"

      # add set rows to data.frame containing other sets for this code
      equivalents_df <- rbind(equivalents_df, set_df)
    }

    # construct label across all sets for this according to the labeller
    # function(s) provided by the user
    if (isTRUE(labels)) {
      i <- equivalents_df$date == max(equivalents_df$date)

      label <- paste(
        equivalents_df$code[i],
        equivalents_df$name[i],
        collapse = ", "
      )

      equivalents_df$label <- label
    } else if (is.list(labels)) {
      label <- lapply(labels, do.call, args = equivalents_df)
      equivalents_df <- cbind(equivalents_df, label)
    }

    # add equivalent sets for this code to the final result
    result <- rbind(result, equivalents_df)
  }

  # remove duplicate rowsl
  result <- unique(result)

  return(result)
}

#' @rdname find_equivalent_codes
#' @export
find_equivalents <- find_equivalent_codes

test_that("find_equivalent_codes returnerer en velformet data.frame", {
  data("klass_131_graph")

  dates <- c("2019-01-01", "2025-01-01")

  resultat <- find_equivalent_codes(
    classification = 131,
    dates = dates,
    graph = klass_131_graph
  )

  expect_s3_class(resultat, "data.frame")
  expect_shape(resultat, ncol = 6)
  expect_contains(
    colnames(resultat),
    c("date", "code", "name", "validFrom", "validTo", "label")
  )

  # slår ut dersom noen av variablene spesifisert under har en NA-verdi
  expect_true(
    all(vapply(
      resultat[c("date", "code", "name", "validFrom", "label")],
      function(col) !all(is.na(col)),
      FUN.VALUE = logical(1)
    ))
  )

  # slår ut dersom noen av datoene mangler ...
  expect_contains(resultat$date, format(as.Date(dates), "%Y"))

  # ... eller resultatet har datoer vi ikke har angitt
  expect_contains(format(as.Date(dates), "%Y"), resultat$date)
})

#' Build a directed graph of code changes based on a Klass classification
#'
#' @param classification The ID of the desired classification.
#'
#' @param date The date which the edges of the graph should be directed towards.
#'
#'   Defaults to the current year plus one, which ensures the graph is directed
#'   to the most recent codes.
#'
#' @return An \code{igraph} object with the vertexes representing codes, and
#'   edges representing changes between codes. The direction of the edges
#'   represent changes towards the date specified in \code{date}.
#'
#' @export
#'
#' @examples
#' library(klassR)
#'
#' # Build a graph directed towards the most recent codes
#' \dontrun{
#' klass_131 <- klass_graph(131)
#' }
#'
#' # Build a graph directed towards valid codes in 2020.
#' \dontrun{
#' klass_131_2020 <- klass_graph(131, "2020-01-01")
#' }
#'
klass_graph <- function(classification, date = NULL) {
  if (is.null(classification)) {
    stop("Please provide a classification ID.")
  }

  ## Downloading codes and code changes

  changes_url <- paste0(
    "https://data.ssb.no/api/klass/v1/classifications/",
    classification,
    "/changes?from=0001-01-01"
  )

  api_endringer <- jsonlite::fromJSON(GetUrl2(changes_url), flatten = TRUE)[[
    "codeChanges"
  ]]

  codes_url <- paste0(
    "https://data.ssb.no/api/klass/v1/classifications/",
    classification,
    "/codes?from=0001-01-01"
  )

  api_alle <- jsonlite::fromJSON(GetUrl2(codes_url), flatten = TRUE)[["codes"]]

  ## Calculating code variants and changes. A code may have more variants than
  ## are present in `api_alle`, since combinations of codes into an already
  ## existing code do not generate a new entry in this table. See `find_dates`.

  variants <-
    do.call(
      rbind,
      lapply(
        unique(api_alle$code),
        find_dates,
        api_alle = api_alle,
        api_endringer = api_endringer
      )
    )

  variants$validFrom <- as.Date(variants$validFrom)
  variants$validTo <- as.Date(variants$validTo)

  variants <- variants[, c(
    "code",
    "name",
    "variant",
    "validFrom",
    "validTo"
  )]

  changes <- api_endringer[, c("oldCode", "changeOccurred", "newCode")]

  changes$variantFrom <- mapply(
    FUN = find_variant_from,
    x = changes$oldCode,
    changeOccurred = changes$changeOccurred,
    MoreArgs = list(variants = variants),
    SIMPLIFY = TRUE
  )

  changes$variantTo <- mapply(
    FUN = find_variant_to,
    x = changes$newCode,
    changeOccurred = changes$changeOccurred,
    MoreArgs = list(variants = variants),
    SIMPLIFY = TRUE
  )

  ## `variant_changes` represents additional changes that are inferred from code
  ## variants where the `validTo` and `validFrom` values are "adjacent", e.g.
  ## variant 1 having a `validTo` of "1964-01-01" and variant 2 having a
  ## `validFrom` of "1964-01-01". In these cases we construct a change between
  ## the two variants on the assumption that the variants are connected.

  variant_changes <-
    do.call(
      rbind,
      lapply(split(variants, variants$code), function(df) {
        # We check if each variant has a `validTo` that matches the next variant's
        # `validFrom`. The last variant always gets `FALSE`, since there is no
        # next variant to check against.
        df$adjacent <- c(df$validTo[-nrow(df)] == df$validFrom[-1], FALSE)

        # We keep rows where `validTo` is equal to a `validFrom` value in another
        # row, or where `validTo` is either the highest value or NA. We wrap this
        # call in suppressWarnings to silence warnings that are due to all values
        # of `validTo` being NA. This is usually the case when there's only one
        # variant of a code.
        suppressWarnings({
          df <- df[
            df$adjacent |
              df$validTo == max(df$validTo, na.rm = TRUE) |
              is.na(df$validTo),
          ]
        })

        # Preparing the changes table. These columns represent the "from" side of
        # the change, so we simply copy the variables already present.
        df$oldCode <- df$code
        df$newCode <- df$code
        df$variantFrom <- df$variant
        df$changeOccurred <- as.character(df$validTo)

        # Constructing the "to" side of the change. We set `variantTo` to the next
        # `variant` if `validTo` matches that variant's `validFrom`. This ensures
        # that we don't connect two variants that aren't adjacent in time, e.g. if
        # a code has been discontinued and then reused after a period of time.
        df$variantTo <- ifelse(df$adjacent, df$variant + 1, NA)

        # We exclude rows where variantTo is NA, since these aren't changes. This
        # also excludes rows representing the last variant of a code (see
        # `max_validTo`). We also only keep the newly constructed columns.
        df <- df[
          !is.na(df$variantTo),
          c(
            "oldCode",
            "changeOccurred",
            "newCode",
            "variantFrom",
            "variantTo"
          )
        ]

        return(df)
      })
    )

  # The final changes table is the combination of changes identified earlier and
  # the changes inferred from evaluating the dates in the variant table. We wrap
  # the call in `unique` in order to remove duplicates, since some of the
  # inferred changes are present in `changes`.
  all_changes <- unique(rbind(changes, variant_changes))

  ## Calculating vertices and edges.

  klass_vertices <- variants

  klass_vertices$vertex <- as.character(1:nrow(klass_vertices))

  all_changes$changeOccurred <- as.Date(all_changes$changeOccurred)

  klass_edges <-
    merge(
      all_changes,
      stats::setNames(
        klass_vertices[, c("code", "variant", "vertex")],
        c("oldCode", "variantFrom", "vertexFrom")
      ),
      by = c("oldCode", "variantFrom"),
      all.x = TRUE
    )

  klass_edges <-
    merge(
      klass_edges,
      stats::setNames(
        klass_vertices[, c("code", "variant", "vertex")],
        c("newCode", "variantTo", "vertexTo")
      ),
      by = c("newCode", "variantTo"),
      all.x = TRUE
    )

  ## Building graph

  graph <-
    igraph::graph_from_edgelist(
      as.matrix(
        klass_edges[, c("vertexFrom", "vertexTo")]
      )
    )

  graph <- igraph::set_edge_attr(
    graph = graph,
    name = "changeOccurred",
    value = klass_edges$changeOccurred
  )

  no_edges <- klass_vertices[
    !klass_vertices$vertex %in% igraph::V(graph)$name,
  ]

  graph <- igraph::add_vertices(
    graph = graph,
    nv = nrow(no_edges),
    attr = list(name = no_edges$vertex)
  )

  # Redirecting edges only if date is not specified.
  # Removes the need to check dates in update_klass_node(), since we can simply follow
  # outgoing edges to reach the code valid at `date`.

  if (!is.null(date)) {
    graph <-
      igraph::reverse_edges(
        graph = graph,
        eids = igraph::E(graph)[igraph::E(graph)$changeOccurred > date]
      )
  }

  klass_vertices <-
    klass_vertices[match(igraph::V(graph)$name, klass_vertices$vertex), ]

  # Building attributes table and applying attributes to vertices. We later
  # extract these attributes when we traverse the graph in update_klass_node()

  attributes <- klass_vertices
  attributes$vertex <- NULL

  names(attributes)[names(attributes) == "name"] <- "label"

  for (i in seq_along(attributes)) {
    graph <-
      igraph::set_vertex_attr(
        graph = graph,
        name = names(attributes)[i],
        value = attributes[, i]
      )
  }

  return(graph)
}

#' Find the variant of a code corresponding to a change *from* a specific code.
#'
#' @param x The code that is being changed
#' @param changeOccurred The date the change occurred
#' @param variants The variants lookup-table.
#'
#' @return The variant corresponding to the code \code{x} at date
#' \code{changeOccurred}.
#'
#' @seealso [find_variant_to]
#'
#' @keywords internal
#'
find_variant_from <- function(x, changeOccurred, variants) {
  variants[
    variants$code == x &
      changeOccurred > variants$validFrom &
      (changeOccurred <= variants$validTo | is.na(variants$validTo)),
  ]$variant
}

#' Find the variant of a code corresponding to a change *to* a specific code.
#'
#' @inheritParams find_variant_from
#'
#' @inherit find_variant_from return
#'
#' @seealso [find_variant_from()]
#'
#' @keywords internal
#'
find_variant_to <- function(x, changeOccurred, variants) {
  variants[
    variants$code == x &
      changeOccurred >= variants$validFrom &
      (changeOccurred < variants$validTo | is.na(variants$validTo)),
  ]$variant
}


#' For a given Klass code, produce a table of dates describing the valid-from
#' and valid-to dates of all versions of the code
#'
#' @param code A Klass code
#' @param api_alle A table of all codes in the classification. See example.
#' @param api_endringer A table of all changes in the classification See
#'   example.
#'
#' @return A \code{data.frame} with number of rows equal to the number of
#'   variants of the combination of code and name, determined by the changes the
#'   code has been involved in. The \code{data.frame} has two columns:
#'
#'   \itemize{
#'    \item{"validFrom"}
#'    \item{"validTo"}
#'   }
#'
#' @keywords internal
#'
find_dates <- function(code, api_alle, api_endringer) {
  dates_df <- api_alle[api_alle$code == code, ]

  dates_df <- dates_df[c(
    "name",
    "validFromInRequestedRange",
    "validToInRequestedRange"
  )]

  names(dates_df)[2:3] <- c("validFrom", "validTo")

  dates_changed <- api_endringer[
    (api_endringer$oldCode == code) | (api_endringer$newCode == code),
  ][["changeOccurred"]]

  if (all(dates_changed %in% unlist(dates_df[c("validFrom", "validTo")]))) {
    dates_df <- dates_df[
      order(dates_df$validFrom),
      c("name", "validFrom", "validTo")
    ]

    dates_df$variant <- 1:nrow(dates_df)
    dates_df$code <- code

    return(dates_df)
  } else {
    dates_df <- dates_df[, c("validFrom", "validTo")]

    # `api_alle` does not give information on codes combining with already
    # existing codes. We use `api_endringer` to expand the dates table to
    # include periods based on when a code has been either combined with
    # another code or a code has split into multiple codes.

    for (date in c(dates_df$validFrom, dates_df$validTo, dates_changed)) {
      if (date %in% c(dates_df$validFrom, dates_df$validTo)) {
        next()
      } else {
        dates_df$rn <- 1:nrow(dates_df)

        row_to_edit <-
          dates_df[
            date >= dates_df$validFrom &
              (date < dates_df$validTo | is.na(dates_df$validTo)),
          ]

        if (nrow(row_to_edit) > 1) {
          stop("Multiple changes matched!")
        }

        old_row <- row_to_edit
        old_row$validTo <- date

        new_row <- row_to_edit
        new_row$validFrom <- date

        dates_df <-
          rbind(
            dates_df[
              !dates_df$rn %in% row_to_edit$rn,
              c("validFrom", "validTo")
            ],
            old_row[, c("validFrom", "validTo")],
            new_row[, c("validFrom", "validTo")]
          )
      }
    }

    dates_df <- dates_df[
      order(dates_df$validFrom),
      c("validFrom", "validTo")
    ]

    dates_df$variant <- 1:nrow(dates_df)

    dates_df$name <-
      mapply(
        find_name,
        validFrom = dates_df$validFrom,
        validTo = dates_df$validTo,
        MoreArgs = list(
          code = code,
          api_alle = api_alle
        ),
        SIMPLIFY = TRUE
      )

    dates_df$code <- code

    return(dates_df[c("code", "name", "variant", "validFrom", "validTo")])
  }
}

#' Find the name of a code valid at a specific date.
#'
#' @inheritParams find_dates
#'
#' @param validFrom The date the code is valid from, in \code{YYYY-MM-DD} format.
#'
#' @param validTo The date the code is valid to, in \code{YYYY-MM-DD} format.
#'
#' @return The name of the code.
#'
#' @keywords internal
#'
find_name <- function(code, validFrom, validTo, api_alle) {
  koder <- api_alle[api_alle$code == code, ]

  if (is.na(validTo)) {
    name <- koder[is.na(koder$validToInRequestedRange), ]$name
  } else {
    name <- koder[
      validFrom >= koder$validFromInRequestedRange &
        (validTo <= koder$validToInRequestedRange |
          is.na(koder$validToInRequestedRange)),
    ][["name"]]
  }

  if (length(name) == 0) {
    return(NA)
  } else {
    return(name)
  }
}

#' Update a code found in a directed graph based on a Klass-classification
#'
#' @inheritParams find_dates
#' @inheritParams klass_node
#'
#' @param output Either a character vector, containing one or more of the items
#'   in the list below, or \code{TRUE} to include all columns.
#'
#'   \describe{
#'    \item{\code{"code"}}{The Klass code.}
#'    \item{\code{"name"}}{The Klass name.}
#'    \item{\code{"validFrom"}}{The date that the code is valid from.}
#'    \item{\code{"validTo"}}{The date that the code is valid to.}
#'    \item{\code{"split"}}{Logical: Does the code split into two or more codes?}
#'    \item{\code{"combined"}}{Logical: Does two or more codes become this code?}
#'    \item{\code{"nextCode"}}{If \code{split == FALSE}, gives the code this code changed into. \code{NA} otherwise.}
#'   }
#'
#' @param combine \code{TRUE} or \code{FALSE}. See the return section.
#'
#' @param report \code{TRUE} or \code{FALSE}. See the return section.
#'
#' @return If \code{report == TRUE} and \code{length(output) > 1 | TRUE}, the
#'   result will be a \code{data.frame} with number of rows equal to the number
#'   of codes in the sequence of changes between the input code and output code.
#'   The columns in the \code{data.frame} are specified with \code{output}.
#'
#'   If \code{report == TRUE} and \code{length(output) == 1}, the result will be
#'   a character vector with length equal to the number of codes in the sequence
#'   of changes between the input code and output code. The contents of the
#'   character vector is specified with \code{output}.
#'
#'   If \code{report == FALSE} and \code{length(output) > 1 | TRUE} the result
#'   will be a \code{data.frame} with one row representing the last code
#'   in the change sequence and columns specified by \code{output}. If a code
#'   has been split, the result will be \code{NA}. If \code{combine == FALSE}
#'   and a code is the result of a combination of codes, the result will be
#'   \code{NA}.
#'
#'   If \code{report == FALSE} and \code{length(output) == 1}, the result will
#'   be a character vector of length one, containing information about the
#'   updated code specified by \code{output}. If a code has been split, the
#'   result will be \code{NA}. If \code{combine == FALSE} and a code is the
#'   result of a combination of codes, the result will be \code{NA}.
#'
#' @seealso See [update_klass] for updating multiple codes in one function call.
#'
#' @keywords internal
update_code <- function(graph,
                        code,
                        date = NA,
                        output = "code",
                        combine = TRUE,
                        report = FALSE) {
  result <- update_klass_node(
    graph = graph,
    node = klass_node(graph, code, date = date)
  )

  nextCodes <- mapply(
    function(node, split) {
      if (split | length(node) == 0) {
        return(NA_character_)
      } else if (length(node) > 1) {
        return(paste0(node$code, collapse = "/"))
      } else {
        return(node$code)
      }
    },
    node = result$nextNodes,
    split = result$split,
    SIMPLIFY = TRUE
  )

  report_df <- data.frame(
    code = result$code,
    name = result$label,
    validFrom = result$validFrom,
    validTo = result$validTo,
    split = result$split,
    combined = result$combined,
    nextCode = unname(nextCodes)
  )

  if (report) {
    return(report_df[, output])
  } else if (any(report_df$split) |
    (!combine & any(report_df$combined)) |
    length(result) == 0) {
    return(NA)
  } else {
    return(report_df[length(result), output])
  }
}

#' Update multiple Klass codes to a desired date.
#'
#' @param codes Codes to be updated.
#'
#' @param dates Optional. Can be used to specify what date each of the codes was
#'   valid in. Supply a character vector of either length 1 to specify the same
#'   valid date for all codes, or of the same length as \code{codes} to specify
#'   valid dates for each code. The character vector(s) should have a format
#'   coercible by \code{\link[base]{as.Date}}, e.g. \code{YYYY-MM-DD}. The
#'   function will return an error if a code was not valid at the specified
#'   date.
#'
#' @param date Optional. Can be used to specify the date the codes should be
#'   updated to, e.g. if you have codes that are valid in year \code{T}, but
#'   want to change the codes to the corresponding version in year \code{T-1}.
#'   If unspecified (the default), the function will update codes to the most
#'   recent version.
#'
#' @param graph Optional. A graph object generated by \code{\link{klass_graph}}.
#'   If you're making multiple calls to [update_klass], you can save some time by
#'   generating the graph beforehand and reusing it for each call to
#'   [update_klass] with this parameter. If providing the graph directly, you do
#'   not need to provide the \code{classification} and \code{date} parameters.
#'
#' @inheritParams klass_graph
#'
#' @inheritParams update_code
#'
#' @return
#'
#'   If \code{output = "code"}, a vector of length \code{length(codes)}
#'   containing either a code if the update is successful or \code{NA} if the
#'   code has been split. If \code{combine = FALSE}, a code being combined with
#'   another code will also return \code{NA}.
#'
#'   If \code{output == TRUE}, a list of length \code{length(codes)} containing
#'   \code{data.frame}s detailing the codes visited through the node search. The
#'   tables have the following columns.
#'
#'   If \code{report == TRUE} and \code{length(output) > 1 | TRUE}, the result
#'   will be a list of \code{data.frame}s with number of rows equal to the
#'   number of codes in the sequence of changes between the input codes and
#'   output codes. The columns in the \code{data.frame}s are specified with
#'   \code{output}.
#'
#'   If \code{report == TRUE} and \code{length(output) == 1}, the result will be
#'   a list of character vectors with length equal to the number of codes in the
#'   sequence of changes between the input code and output code. The contents of
#'   the character vectors is specified with \code{output}.
#'
#'   If \code{report == FALSE} and \code{length(output) > 1 | TRUE} the result
#'   will be a list of \code{data.frame}s with one row representing the last
#'   code in the change sequence and columns specified by \code{output}. If a
#'   code has been split, the result will be \code{NA}. If \code{combine ==
#'   FALSE} and a code is the result of a combination of codes, the result will
#'   be \code{NA}.
#'
#'   If \code{report == FALSE} and \code{length(output) == 1}, the result will
#'   be a character vector containing information about the updated codes
#'   specified by \code{output}. If a code has been split, the result will be
#'   \code{NA}. If \code{combine == FALSE} and a code is the result of a
#'   combination of codes, the result will be \code{NA}.
#'
#'
#' @export
#'
#' @examples
#' library(klassR)
#' codes <- get_klass(131, date = "2020-01-01")[["code"]]
#'
#' \dontrun{
#' updated_codes <- update_klass(codes,
#'   dates = "2020-01-01",
#'   classification = 131
#' )
#' }
#'
update_klass <- function(codes,
                         dates = NA,
                         classification = NULL,
                         date = NULL,
                         graph = klass_graph(classification, date),
                         output = "code",
                         report = FALSE,
                         combine = TRUE) {
  if (!methods::hasArg(graph) & !methods::hasArg(classification)) {
    stop(
      "\nPlease provide either:\n",
      "- A graph with the `graph` argument\n",
      "- A classification ID with the `classification` argument"
    )
  }

  # TODO Optimize by making a table of input codes, updating those, and
  # returning an expanded vector based on input

  simplify <- length(output) == 1 & !isTRUE(output) & !report

  result <- mapply(
    FUN = update_code,
    code = codes,
    date = dates,
    SIMPLIFY = simplify,
    MoreArgs = list(
      graph = graph,
      output = output,
      combine = combine,
      report = report
    )
  )

  return(result)
}

test_that("update_klass gir riktig resultat ved enkle endringer", {
  data("klass_131_2020_graph")
  graph <- klass_131_2020_graph
  data("api_endringer_2019")

  endringer_kommunestruktur_enkle <-
    api_endringer_2019 %>%
    # 2020 hadde mange enkle endringer
    dplyr::filter(changeOccurred == "2020-01-01") %>%
    dplyr::group_by(oldCode) %>%
    dplyr::filter(dplyr::n() == 1) %>% # vi tester ikke delinger av koder
    dplyr::group_by(newCode) %>%
    dplyr::filter(dplyr::n() == 1) %>% # vi tester ikke sammenslåtte koder
    dplyr::ungroup() %>%
    dplyr::select(oldCode, newCode, changeOccurred)

  omkodet <-
    endringer_kommunestruktur_enkle %>%
    dplyr::mutate(
      oppdatert = update_klass(
        codes = oldCode,
        classification = 131,
        output = "code",
        report = FALSE,
        graph = graph
      )
    )

  feil <-
    omkodet %>%
    dplyr::rowwise() %>%
    dplyr::filter(!newCode %in% oppdatert)

  expect_equal(nrow(feil), 0)
})

test_that("update_klass gir riktig resultat ved sammenslåtte koder", {
  data("klass_131_1964_graph")
  graph <- klass_131_1964_graph

  data("api_endringer_1963")

  endringer_kommunestruktur_sammenslåinger <-
    api_endringer_1963 %>%
    dplyr::filter(changeOccurred == "1964-01-01") %>%
    dplyr::group_by(oldCode, changeOccurred) %>%
    dplyr::filter(dplyr::n() == 1) %>% # vi tester ikke delinger av koder
    dplyr::group_by(newCode) %>%
    dplyr::filter(dplyr::n() > 1)

  omkodet <-
    endringer_kommunestruktur_sammenslåinger %>%
    dplyr::mutate(
      oppdatert = update_klass(
        codes = oldCode,
        dates = as.Date(changeOccurred) - 1,
        classification = 131,
        graph = graph
      ),
      oppdatert_ikkecomb = update_klass(
        codes = oldCode,
        dates = as.Date(changeOccurred) - 1,
        classification = 131,
        graph = graph,
        combine = FALSE
      )
    )

  omkodet %>%
    dplyr::rowwise() %>%
    dplyr::filter(!newCode %in% oppdatert) %>%
    # `1441 Selje` is split in 1964 to `1441 Selje` and `1439 Vågsøy`,
    # but only `1439 Vågsøy` is recorded in the changes API
    dplyr::filter(!oldCode == "1441" & changeOccurred == "1964-01-01") %>%
    nrow() %>%
    expect_equal(expected = 0)

  omkodet %>%
    dplyr::rowwise() %>%
    dplyr::filter(!is.na(oppdatert_ikkecomb)) %>%
    nrow() %>%
    expect_equal(expected = 0)
})

test_that("update_klass gir riktig resultat ved ugyldige koder", {
  data("klass_131_graph")
  graph <- klass_131_graph

  expect_true(
    all(
      is.na(
        update_klass(c("foo", "bar", "egg", "ham"), graph = graph)
      )
    )
  )
})

test_that("update_klass gir riktig resultat ved delte koder", {
  data("klass_131_1964_graph")
  graph <- klass_131_1964_graph

  data("api_endringer_1963")

  endringer_kommunestruktur_delinger <-
    api_endringer_1963 %>%
    # 1964 hadde klart flest delinger av koder
    dplyr::filter(changeOccurred == "1964-01-01") %>%
    dplyr::group_by(oldCode) %>%
    dplyr::filter(dplyr::n() > 1) %>%
    dplyr::ungroup() %>%
    dplyr::select(oldCode, newCode, changeOccurred)

  omkodet <-
    endringer_kommunestruktur_delinger %>%
    dplyr::mutate(
      oppdatert = update_klass(
        codes = oldCode,
        dates = as.Date(changeOccurred) - 1,
        classification = 131,
        graph = graph
      )
    )

  expect_true(all(is.na(omkodet$oppdatert)))
})


test_that("update_klass gir forventet format på output", {
  data("klass_131_graph")
  graph <- klass_131_graph

  update_helper <- function(output, report) {
    update_klass(
      codes = "0301",
      dates = "1838-01-01",
      output = output,
      report = report,
      graph = graph
    )
  }

  expect_type(update_helper(output = "code", report = FALSE), "character")
  expect_type(update_helper(output = "code", report = TRUE), "list")
  expect_type(update_helper(output = TRUE, report = FALSE), "list")
  expect_type(update_helper(output = "code", report = TRUE)[[1]], "character")
  expect_s3_class(
    update_helper(output = c("code", "name"), report = TRUE)[[1]],
    "data.frame"
  )
  expect_type(update_helper(output = c("code", "name"), report = TRUE), "list")
  expect_type(update_helper(output = TRUE, report = FALSE), "list")
  expect_s3_class(
    update_helper(output = TRUE, report = FALSE)[[1]],
    "data.frame"
  )
  expect_equal(nrow(update_helper(output = TRUE, report = FALSE)[[1]]), 1)
  expect_equal(
    nrow(update_helper(output = c("code", "name"), report = FALSE)[[1]]),
    1
  )
  expect_equal(length(update_helper(output = "code", report = FALSE)), 1)
})

test_that("update_klass oppdaterer koder som har hatt navneendringer", {
  ## ... og der navneendringene ikke er logget som endringer, jf. [#56]

  data("klass_131_graph")
  graph <- klass_131_graph

  expect_equal(
    update_klass(
      codes = "1420",
      dates = "1838-01-01",
      graph = graph
    ),
    "4640",
    ignore_attr = TRUE
  )
})

# get_klass gives an informative error message when a correspondence is missing

    Code
      get_klass(131, correspond = 556, date = "2020-01-01")
    Message
      Connection failed with error code 404:
      Classification 'Standard for kommuneinndeling' has no correspondence table with Classification 'Kodeliste for avslutning av kvalifiseringsprogram, KVP'
    Condition
      Error in `stop_quietly()`:

# get_klass gives an informative error message when a date is too old

    Code
      get_klass(131, date = "1600-01-01")
    Condition
      Error in `get_klass()`:
      ! No codes were found for classification 131 with the current search parameters. The specified date (1600-01-01) may be too early.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UpdateKlass.R
\name{update_klass}
\alias{update_klass}
\title{Update multiple Klass codes to a desired date.}
\usage{
update_klass(
  codes,
  dates = NA,
  classification = NULL,
  date = NULL,
  graph = klass_graph(classification, date),
  output = "code",
  report = FALSE,
  combine = TRUE
)
}
\arguments{
\item{codes}{Codes to be updated.}

\item{dates}{Optional. Can be used to specify what date each of the codes was
valid in. Supply a character vector of either length 1 to specify the same
valid date for all codes, or of the same length as \code{codes} to specify
valid dates for each code. The character vector(s) should have a format
coercible by \code{\link[base]{as.Date}}, e.g. \code{YYYY-MM-DD}. The
function will return an error if a code was not valid at the specified
date.}

\item{classification}{The ID of the desired classification.}

\item{date}{Optional. Can be used to specify the date the codes should be
updated to, e.g. if you have codes that are valid in year \code{T}, but
want to change the codes to the corresponding version in year \code{T-1}.
If unspecified (the default), the function will update codes to the most
recent version.}

\item{graph}{Optional. A graph object generated by \code{\link{klass_graph}}.
If you're making multiple calls to \link{update_klass}, you can save some time by
generating the graph beforehand and reusing it for each call to
\link{update_klass} with this parameter. If providing the graph directly, you do
not need to provide the \code{classification} and \code{date} parameters.}

\item{output}{Either a character vector, containing one or more of the items
in the list below, or \code{TRUE} to include all columns.

\describe{
\item{\code{"code"}}{The Klass code.}
\item{\code{"name"}}{The Klass name.}
\item{\code{"validFrom"}}{The date that the code is valid from.}
\item{\code{"validTo"}}{The date that the code is valid to.}
\item{\code{"split"}}{Logical: Does the code split into two or more codes?}
\item{\code{"combined"}}{Logical: Does two or more codes become this code?}
\item{\code{"nextCode"}}{If \code{split == FALSE}, gives the code this code changed into. \code{NA} otherwise.}
}}

\item{report}{\code{TRUE} or \code{FALSE}. See the return section.}

\item{combine}{\code{TRUE} or \code{FALSE}. See the return section.}
}
\value{
If \code{output = "code"}, a vector of length \code{length(codes)}
containing either a code if the update is successful or \code{NA} if the
code has been split. If \code{combine = FALSE}, a code being combined with
another code will also return \code{NA}.

If \code{output == TRUE}, a list of length \code{length(codes)} containing
\code{data.frame}s detailing the codes visited through the node search. The
tables have the following columns.

If \code{report == TRUE} and \code{length(output) > 1 | TRUE}, the result
will be a list of \code{data.frame}s with number of rows equal to the
number of codes in the sequence of changes between the input codes and
output codes. The columns in the \code{data.frame}s are specified with
\code{output}.

If \code{report == TRUE} and \code{length(output) == 1}, the result will be
a list of character vectors with length equal to the number of codes in the
sequence of changes between the input code and output code. The contents of
the character vectors is specified with \code{output}.

If \code{report == FALSE} and \code{length(output) > 1 | TRUE} the result
will be a list of \code{data.frame}s with one row representing the last
code in the change sequence and columns specified by \code{output}. If a
code has been split, the result will be \code{NA}. If \code{combine ==
  FALSE} and a code is the result of a combination of codes, the result will
be \code{NA}.

If \code{report == FALSE} and \code{length(output) == 1}, the result will
be a character vector containing information about the updated codes
specified by \code{output}. If a code has been split, the result will be
\code{NA}. If \code{combine == FALSE} and a code is the result of a
combination of codes, the result will be \code{NA}.
}
\description{
Update multiple Klass codes to a desired date.
}
\examples{
library(klassR)
codes <- get_klass(131, date = "2020-01-01")[["code"]]

\dontrun{
updated_codes <- update_klass(codes,
  dates = "2020-01-01",
  classification = 131
)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Klass_list.R
\name{GetBaseUrl}
\alias{GetBaseUrl}
\title{Get base address for fetching classifications from}
\usage{
GetBaseUrl()
}
\value{
String with address
}
\description{
Get base address for fetching classifications from
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Hent_data.R
\name{MakeUrl}
\alias{MakeUrl}
\title{Internal function to create URL address}
\usage{
MakeUrl(
  classification,
  correspond = NULL,
  correspondID = NULL,
  variant_name = NULL,
  type = "vanlig",
  fratil = FALSE,
  date = NULL,
  output_level_coding = NULL,
  language_coding = NULL
)
}
\arguments{
\item{classification}{Classification number}

\item{correspond}{Target number for correspondence table}

\item{variant_name}{The name of the variant of the classification}

\item{type}{String describing type. "vanlig" for normal classification and "kor" for correspondence. Default = "vanlig"}

\item{fratil}{True/False for whether a date interval is to be used. Default = False}

\item{date}{Date(s) for classification}

\item{output_level_coding}{Coding for output level}

\item{language_coding}{Coding for language}
}
\value{
String url adress
}
\description{
Internal function to create URL address
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format.R
\name{formattering}
\alias{formattering}
\title{Format vector for industry codes}
\usage{
formattering(x, classification)
}
\arguments{
\item{x}{\itemize{
\item vector of character
}}

\item{classification}{\itemize{
\item classification number
}}
}
\value{
vector of character
}
\description{
Format vector for industry codes
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Levels.R
\name{Levels}
\alias{Levels}
\title{Title}
\usage{
Levels(input_level, output_level, klass_data)
}
\arguments{
\item{input_level}{Classification level from the original dataset.}

\item{output_level}{Classification level for which the codes and names are desired by the user.}

\item{klass_data}{Classification file retrieved from KLASS.}
}
\value{
Data frame with the input and extra desired classification levels. Includes codes and names for each level.
}
\description{
Title
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{klass_131_2020_graph}
\alias{klass_131_2020_graph}
\title{Test Graph data for municipalities in 2020}
\format{
An object of class \code{igraph} of length 2000.
}
\usage{
klass_131_2020_graph
}
\description{
A nested list of graph data for using in testing
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UpdateKlass.R
\name{update_code}
\alias{update_code}
\title{Update a code found in a directed graph based on a Klass-classification}
\usage{
update_code(
  graph,
  code,
  date = NA,
  output = "code",
  combine = TRUE,
  report = FALSE
)
}
\arguments{
\item{graph}{A graph generated by \code{\link{klass_graph}}.}

\item{code}{A Klass code}

\item{date}{Optional. The specific date the supplied code is valid in.}

\item{output}{Either a character vector, containing one or more of the items
in the list below, or \code{TRUE} to include all columns.

\describe{
\item{\code{"code"}}{The Klass code.}
\item{\code{"name"}}{The Klass name.}
\item{\code{"validFrom"}}{The date that the code is valid from.}
\item{\code{"validTo"}}{The date that the code is valid to.}
\item{\code{"split"}}{Logical: Does the code split into two or more codes?}
\item{\code{"combined"}}{Logical: Does two or more codes become this code?}
\item{\code{"nextCode"}}{If \code{split == FALSE}, gives the code this code changed into. \code{NA} otherwise.}
}}

\item{combine}{\code{TRUE} or \code{FALSE}. See the return section.}

\item{report}{\code{TRUE} or \code{FALSE}. See the return section.}
}
\value{
If \code{report == TRUE} and \code{length(output) > 1 | TRUE}, the
result will be a \code{data.frame} with number of rows equal to the number
of codes in the sequence of changes between the input code and output code.
The columns in the \code{data.frame} are specified with \code{output}.

If \code{report == TRUE} and \code{length(output) == 1}, the result will be
a character vector with length equal to the number of codes in the sequence
of changes between the input code and output code. The contents of the
character vector is specified with \code{output}.

If \code{report == FALSE} and \code{length(output) > 1 | TRUE} the result
will be a \code{data.frame} with one row representing the last code
in the change sequence and columns specified by \code{output}. If a code
has been split, the result will be \code{NA}. If \code{combine == FALSE}
and a code is the result of a combination of codes, the result will be
\code{NA}.

If \code{report == FALSE} and \code{length(output) == 1}, the result will
be a character vector of length one, containing information about the
updated code specified by \code{output}. If a code has been split, the
result will be \code{NA}. If \code{combine == FALSE} and a code is the
result of a combination of codes, the result will be \code{NA}.
}
\description{
Update a code found in a directed graph based on a Klass-classification
}
\seealso{
See \link{update_klass} for updating multiple codes in one function call.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Klass_list.R
\name{get_name}
\alias{get_name}
\alias{GetName}
\title{Get the name of a classification version}
\usage{
get_name(version)

GetName(version)
}
\arguments{
\item{version}{Version number}
}
\value{
string or vector of strings with name of version
}
\description{
Get the name of a classification version
}
\examples{
get_name("33")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{api_endringer_1963}
\alias{api_endringer_1963}
\title{Test data for changes in municipality codes from 1963 onwards}
\format{
An object of class \code{data.frame} with 1172 rows and 7 columns.
}
\usage{
api_endringer_1963
}
\description{
A data.frame containing information about changes in municipality codes
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Klass_list.R
\name{get_version}
\alias{get_version}
\alias{GetVersion}
\title{Get version number of a class given a date}
\usage{
get_version(classification = NULL, date = NULL, family = NULL, klassNr = FALSE)

GetVersion(klass = NULL, date = NULL, family = NULL, klassNr = FALSE)
}
\arguments{
\item{classification}{Classification number}

\item{date}{Date for version to be valid}

\item{family}{Family ID number if a list of version number for all classes is desired}

\item{klassNr}{True/False for whether to output classification numbers. Default = FALSE}

\item{klass}{Deprecated; use \code{classification} instead.}
}
\value{
Number, vector or data frame with version numbers and calssification numbers if specified.
}
\description{
Get version number of a class given a date
}
\examples{
get_version(7)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Klass_list.R
\name{GetNums}
\alias{GetNums}
\title{Get target ID numbers from Url}
\usage{
GetNums(x)
}
\arguments{
\item{x}{Url address}
}
\value{
Number
}
\description{
Get target ID numbers from Url
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Klass_list.R
\name{correspond_list}
\alias{correspond_list}
\alias{CorrespondList}
\title{Correspondence list
Print a list of correspondence tables for a given classification with source and target IDs}
\usage{
correspond_list(classification, date = NULL)

CorrespondList(klass, date = NULL)
}
\arguments{
\item{classification}{Classification number}

\item{date}{Date for classification (format = "YYYY-mm-dd"). Default is current date}

\item{klass}{Deprecated; use \code{classification} instead.}
}
\value{
Data frame with list of corrsepondence tables, source ID and target ID.
}
\description{
Correspondence list
Print a list of correspondence tables for a given classification with source and target IDs
}
\examples{
\donttest{
correspond_list("7")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UpdateKlass-graph-build.R
\name{find_dates}
\alias{find_dates}
\title{For a given Klass code, produce a table of dates describing the valid-from
and valid-to dates of all versions of the code}
\usage{
find_dates(code, api_alle, api_endringer)
}
\arguments{
\item{code}{A Klass code}

\item{api_alle}{A table of all codes in the classification. See example.}

\item{api_endringer}{A table of all changes in the classification See
example.}
}
\value{
A \code{data.frame} with number of rows equal to the number of
variants of the combination of code and name, determined by the changes the
code has been involved in. The \code{data.frame} has two columns:

\itemize{
\item{"validFrom"}
\item{"validTo"}
}
}
\description{
For a given Klass code, produce a table of dates describing the valid-from
and valid-to dates of all versions of the code
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UpdateKlass-graph-navigate.R
\name{is_combined}
\alias{is_combined}
\title{Given a graph and a node, determine if the node is a result of combinations
of multiple codes.}
\usage{
is_combined(graph, node, compare_node = NULL)
}
\arguments{
\item{graph}{A graph generated by \code{\link{klass_graph}}.}

\item{node}{A node as returned by \code{\link{klass_node}} or
\code{\link[igraph]{V}}.}

\item{compare_node}{Optional. A node to compare \code{node} with when
determining whether \code{node} is combined. See details.}
}
\value{
\code{TRUE} if the node is a combination of two or more nodes,
otherwise \code{FALSE}.
}
\description{
Given a graph and a node, determine if the node is a result of combinations
of multiple codes.
}
\details{
The function will attempt to reconcile nodes that have split and
then later merged again when evaluating a node's combinedness.

If \code{compare_node == NULL}, a node is considered to be combined if more
than one node that does not itself have a parent (i.e. codes at the start
of a sequence of changes) contribute to \code{node}.

If \code{compare_node != NULL}, a node is considered to be combined if any
node that is not an ancestor of \code{compare_node} contributes to
\code{node}, i.e. all paths from \code{node} to the parents of \code{node}
pass through \code{compare_node}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Hent_data.R
\name{get_variant_name}
\alias{get_variant_name}
\title{Get variant name
Internal function for fetching the variant name based on the number}
\usage{
get_variant_name(variant)
}
\arguments{
\item{variant}{The variant number}
}
\description{
Get variant name
Internal function for fetching the variant name based on the number
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Hent_data.R
\name{CheckDate}
\alias{CheckDate}
\title{Internal function to check date}
\usage{
CheckDate(date)
}
\arguments{
\item{date}{Date}
}
\description{
Internal function to check date
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UpdateKlass-graph-navigate.R
\name{count_neighbors}
\alias{count_neighbors}
\title{Count the neighbors of a node.}
\usage{
count_neighbors(graph, node, mode)
}
\arguments{
\item{graph}{A graph generated by \code{\link{klass_graph}}.}

\item{node}{A node as returned by \code{\link{klass_node}} or
\code{\link[igraph]{V}}.}

\item{mode}{Whether to query outgoing (\sQuote{out}), incoming
(\sQuote{in}) edges, or both types (\sQuote{all}). This is
ignored for undirected graphs.}
}
\value{
A numeric vector of length one giving the number of neighbors.
}
\description{
Count the neighbors of a node.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Hent_data.R
\name{stop_quietly}
\alias{stop_quietly}
\title{Stop quietly function
Stop from a function without an error. Used for stopping when no internet}
\usage{
stop_quietly()
}
\description{
Stop quietly function
Stop from a function without an error. Used for stopping when no internet
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/KLASS.R
\name{apply_klass}
\alias{apply_klass}
\alias{ApplyKlass}
\title{Match and convert a classification}
\usage{
apply_klass(
  x,
  classification,
  date = NULL,
  variant = NULL,
  correspond = NULL,
  language = "nb",
  output_level = NULL,
  output = "name",
  format = TRUE
)

ApplyKlass(
  x,
  klass,
  date = NULL,
  variant = NULL,
  correspond = NULL,
  language = "nb",
  output_level = NULL,
  output = "name",
  format = TRUE
)
}
\arguments{
\item{x}{Input vector of classification codes. Vector must match "code" column from a call to get_klass().}

\item{classification}{Classification number}

\item{date}{String for the required date of the classification. Format must be "yyyy-mm-dd". For an inverval, provide two dates as a vector. If blank, will default to today's date.}

\item{variant}{The classification variant to fetch (if a variant is wanted).}

\item{correspond}{ID number for target in correspondence table. For correspondence between two dates within the same classification, use correspond = TRUE.}

\item{language}{Default "nb" for Norwegian (Bokmål). Also "nn" (Nynorsk) and "en" (English available for some classifications)}

\item{output_level}{Desired output level}

\item{output}{String describing output. May be "name" (default), "code" or "both".}

\item{format}{Logical for whther to run formatting av input vector x (Default = TRUE), important to check if formatting is in one level.}

\item{klass}{Deprecated; use \code{classification} instead.}
}
\value{
A vector or data frame is returned with names and/or code of the desired output level.
}
\description{
Match and convert a classification
}
\examples{
data(klassdata)
kommune_names <- apply_klass(
  x = klassdata$kommune,
  classification = 131,
  language = "en",
  format = FALSE
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{klassdata}
\alias{klassdata}
\title{Testdata for klassR package}
\format{
A data frame containing 100 rows and 7 variables:
\describe{
\item{ID}{Identification number}
\item{sex}{1/2 variable for sex}
\item{education}{4-digit number for education standard ISCED97 (level and subject area) NUS (klass = 66) 2015.01.01}
\item{kommune}{4-digit code for Norwegian municipality (klass = 131). Based on 2015.01.01}
\item{kommune2}{Numeric variable for Norwegian municipality with dropped leading zero's for testing (klass = 131). Based on 2015.01.01}
\item{nace5}{5-digit code for industry (NACE). Based on 01.01.2015 standard industry codes (klass = 7)}
\item{occupation}{4-digit occupation codes using standard for STYRK-08 (klass = 7) 2015.01.01}
}
}
\usage{
klassdata
}
\description{
A dataset containing variables for testing of Statistics Norways classification API with the klassR package. Some observations are missing or incorrect for testing and demonstrations.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format.R
\name{formattering_nace}
\alias{formattering_nace}
\title{Format vector for industry codes}
\usage{
formattering_nace(x)
}
\arguments{
\item{x}{Character vector}
}
\value{
Formatted charcter vector
}
\description{
Format vector for industry codes
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkLevel.R
\name{levelCheck}
\alias{levelCheck}
\title{Check which level the input data/vector is}
\usage{
levelCheck(x, klass_data)
}
\arguments{
\item{x}{\itemize{
\item vector of character
}}

\item{klass_data}{\itemize{
\item 
\itemize{
\item the right formatting to the classification levels
}
}}
}
\value{
The hierarchical level of the input data is returned.
}
\description{
The input level is calculated as the mode (most common) hierarchical
level in the input vector.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Klass_list.R
\name{list_klass}
\alias{list_klass}
\alias{ListKlass}
\title{Classification list
Get a full list of all classifications and codelists}
\usage{
list_klass(codelists = FALSE, language = "nb")

ListKlass(codelists = FALSE, language = "nb")
}
\arguments{
\item{codelists}{True/False for whether to include codelists. Default = FALSE}

\item{language}{Two letter string for the requested language output. Default is Bokmål ("nb"). Nynorsk ("nn") and English ("en").}
}
\value{
A data frame containing a full list of classifications. The data frame includes the classification name, number, family and type.
}
\description{
Classification list
Get a full list of all classifications and codelists
}
\examples{
head(list_klass(codelists = TRUE))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Klass_list.R
\name{GetUrl}
\alias{GetUrl}
\title{Get json file from Url}
\usage{
GetUrl(url)
}
\arguments{
\item{url}{String url address}
}
\value{
text in json format
}
\description{
Get json file from Url
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Hent_data.R
\name{check_connect}
\alias{check_connect}
\title{Check connection
Function to check that a connection to data.ssb.no is able to be established}
\usage{
check_connect(url)
}
\arguments{
\item{url}{String url address for connection to check}
}
\value{
Nothing is returned but a error or warning message is return if no connection is available
}
\description{
Check connection
Function to check that a connection to data.ssb.no is able to be established
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UpdateKlass-graph-navigate.R
\name{klass_node}
\alias{klass_node}
\title{Given a Klass graph, find the node corresponding to a code and (optionally) a
date.}
\usage{
klass_node(graph, x, date = NA)
}
\arguments{
\item{graph}{A graph generated by \code{\link{klass_graph}}.}

\item{x}{The code to search for.}

\item{date}{Optional. The specific date the supplied code is valid in.}
}
\value{
The node in the graph corresponding to the supplied code. If date is
not provided, the node with the most recent code is returned. If date is
provided, the code with date between \code{validFrom} and \code{validTo} is
returned.
}
\description{
Given a Klass graph, find the node corresponding to a code and (optionally) a
date.
}
\examples{
\dontrun{
# Build a graph directed towards the most recent codes.
library(klassR)
klass_131 <- klass_graph(131)

# Find the most recent node in the graph representing the code "0101" (Halden,
# valid to 2020.)
halden_node <- klass_node(klass_131, "0101")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UpdateKlass-graph-navigate.R
\name{is_split}
\alias{is_split}
\title{Given a graph and a node, determine if the node is a split code.}
\usage{
is_split(graph, node)
}
\arguments{
\item{graph}{A graph generated by \code{\link{klass_graph}}.}

\item{node}{A node as returned by \code{\link{klass_node}} or
\code{\link[igraph]{V}}.}
}
\value{
\code{TRUE} if the node is split, otherwise \code{FALSE}.
}
\description{
Given a graph and a node, determine if the node is a split code.
}
\details{
The function will attempt to reconcile nodes that have split and
then later merged again. A node is considered to be split if there is more
than one node that does not itself have children (i.e. nodes at the end of
a sequence of changes) that can be reached from \code{node}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Hent_data.R
\name{get_klass}
\alias{get_klass}
\alias{GetKlass}
\title{Fetch Statistics Norway classification data using API}
\usage{
get_klass(
  classification,
  date = NULL,
  correspond = NULL,
  correspondID = NULL,
  variant = NULL,
  output_level = NULL,
  language = "nb",
  output_style = "normal",
  notes = FALSE,
  quiet = TRUE
)

GetKlass(
  klass,
  date = NULL,
  correspond = NULL,
  correspondID = NULL,
  variant = NULL,
  output_level = NULL,
  language = "nb",
  output_style = "normal",
  notes = FALSE,
  quiet = TRUE
)
}
\arguments{
\item{classification}{Number/string of the classification ID/number. (use klass_list() to find this)}

\item{date}{String for the required date of the classification. Format must be "yyyy-mm-dd". For an inverval, provide two dates as a vector. If blank, will default to today's date.}

\item{correspond}{Number/string of the target classification for correspondence table (if a correspondence table is requested).}

\item{correspondID}{ID number of the correspondence table to retrieve. Use as an alternative to correspond.}

\item{variant}{The classification variant to fetch (if a variant is wanted).}

\item{output_level}{Number/string specifying the requested hierarchy level (optional).}

\item{language}{Two letter string for the requested language output. Default is Bokmål ("nb"). Nynorsk ("nn") and English ("en") also available for some classification.)}

\item{output_style}{String variable for the output type. Default is "normal". Specify "wide" for a wide formatted table output.}

\item{notes}{Logical for if notes should be returned as a column. Default FALSE}

\item{quiet}{Logical for whether to suppress the printing of the API address. Default TRUE.}

\item{klass}{Deprecated; use \code{classification} instead.}
}
\value{
The function returns a data frame of the specified classification/correspondence table. Output variables include:
code, parentCode, level, and name for standard lists. For correspondence tables variables include:
sourceCode, sourceName, targetCode and targetName. For date correspondence tables variables include:
oldCode, oldName, newCode and newName. For "wide" output, code and name with level suffixes is specified.
For date ranges, validFromInRequestedRange and validToInRequestedRange give the dates for the classification. Variable ChangeOccured gives the
effective date for classification change in classification change tables.
}
\description{
Fetch Statistics Norway classification data using API
}
\examples{
# Get classification for occupation classifications
head(get_klass(classification = "7"))
# Get classification for occupation classifications in English
head(get_klass(classification = "7", language = "en"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Klass_list.R
\name{get_family}
\alias{get_family}
\alias{GetFamily}
\title{Identify corresponding family from a classification number}
\usage{
get_family(classification)

GetFamily(klass)
}
\arguments{
\item{classification}{Classification number}

\item{klass}{Deprecated; use \code{classification} instead.}
}
\value{
Family number
}
\description{
Identify corresponding family from a classification number
}
\examples{
get_family(classification = 7)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/KLASS.R
\name{MakeChar}
\alias{MakeChar}
\title{Conversion to character}
\usage{
MakeChar(x)
}
\arguments{
\item{x}{a number or vector of numbers}
}
\value{
x converted to a string or vector of strings.
}
\description{
Conversion to character
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UpdateKlass-graph-build.R
\name{find_variant_to}
\alias{find_variant_to}
\title{Find the variant of a code corresponding to a change \emph{to} a specific code.}
\usage{
find_variant_to(x, changeOccurred, variants)
}
\arguments{
\item{x}{The code that is being changed}

\item{changeOccurred}{The date the change occurred}

\item{variants}{The variants lookup-table.}
}
\value{
The variant corresponding to the code \code{x} at date
\code{changeOccurred}.
}
\description{
Find the variant of a code corresponding to a change \emph{to} a specific code.
}
\seealso{
\code{\link[=find_variant_from]{find_variant_from()}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UpdateKlass-graph-build.R
\name{find_name}
\alias{find_name}
\title{Find the name of a code valid at a specific date.}
\usage{
find_name(code, validFrom, validTo, api_alle)
}
\arguments{
\item{code}{A Klass code}

\item{validFrom}{The date the code is valid from, in \code{YYYY-MM-DD} format.}

\item{validTo}{The date the code is valid to, in \code{YYYY-MM-DD} format.}

\item{api_alle}{A table of all codes in the classification. See example.}
}
\value{
The name of the code.
}
\description{
Find the name of a code valid at a specific date.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Convert_table.R
\name{ConvertTable}
\alias{ConvertTable}
\title{Code level convert table (internal function)}
\usage{
ConvertTable(
  klass_data,
  code = "code",
  parentcode = "parentCode",
  level = "level"
)
}
\arguments{
\item{klass_data}{Klass data frame to convert}

\item{code}{Name of variable containing code (Default = "code")}

\item{parentcode}{Name of variable of parent code (Default = "parentCode")}

\item{level}{Name of variable containing level (Default = "level")}
}
\value{
A dataframe
}
\description{
Code level convert table (internal function)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UpdateKlass-graph-build.R
\name{klass_graph}
\alias{klass_graph}
\title{Build a directed graph of code changes based on a Klass classification}
\usage{
klass_graph(classification, date = NULL)
}
\arguments{
\item{classification}{The ID of the desired classification.}

\item{date}{The date which the edges of the graph should be directed towards.

Defaults to the current year plus one, which ensures the graph is directed
to the most recent codes.}
}
\value{
An \code{igraph} object with the vertexes representing codes, and
edges representing changes between codes. The direction of the edges
represent changes towards the date specified in \code{date}.
}
\description{
Build a directed graph of code changes based on a Klass classification
}
\examples{
library(klassR)

# Build a graph directed towards the most recent codes
\dontrun{
klass_131 <- klass_graph(131)
}

# Build a graph directed towards valid codes in 2020.
\dontrun{
klass_131_2020 <- klass_graph(131, "2020-01-01")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_equivalents.R
\name{find_equivalent_nodes}
\alias{find_equivalent_nodes}
\title{Find the equivalent sets of a node at various dates}
\usage{
find_equivalent_nodes(node, dates, graph)
}
\arguments{
\item{node}{The node that we're finding the equivalent sets of}

\item{dates}{The dates that we want to find the equivalent sets in}

\item{graph}{The graph that the nodes come from}
}
\value{
A named list of \code{length(dates)} containing the equivalent nodes for
each date. The names of the returned list are the dates provided in \code{date},
coerced to character.
}
\description{
Find the equivalent sets of a node at various dates
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UpdateKlass-graph-navigate.R
\name{update_klass_node}
\alias{update_klass_node}
\title{Given a node and a graph, find the node at the end of a sequence of changes.}
\usage{
update_klass_node(graph, node)
}
\arguments{
\item{graph}{A graph generated by \code{\link{klass_graph}}.}

\item{node}{A node as returned by \code{\link{klass_node}} or
\code{\link[igraph]{V}}.}
}
\value{
A sequence of vertices, starting with \code{node} and ending with the
last visited node.
}
\description{
Given a node and a graph, find the node at the end of a sequence of changes.
}
\examples{
\dontrun{
# Build a graph directed towards the most recent codes.
library(klassR)
klass_131 <- klass_graph(131)

# Find the most recent node in the graph representing the code "0101" (Halden,
# valid to 2020.)
halden_node <- klass_node(klass_131, "0101")

# Find the most recent code corresponding to 0101 Halden
halden_node_updated <- update_klass_node(klass_131, halden_node)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_equivalents.R
\name{find_equivalent_codes}
\alias{find_equivalent_codes}
\alias{find_equivalents}
\title{Find equivalent sets of codes in a Klass classification}
\usage{
find_equivalent_codes(
  classification,
  dates,
  labels = TRUE,
  graph = klass_graph(classification),
  date_format = "\%Y"
)

find_equivalents(
  classification,
  dates,
  labels = TRUE,
  graph = klass_graph(classification),
  date_format = "\%Y"
)
}
\arguments{
\item{classification}{The Klass classification to be used}

\item{dates}{The dates that equivalent sets of codes should be found for.}

\item{labels}{\code{TRUE}, \code{FALSE} or a named list of functions.

This parameter controls whether or not to add group labels to groups of
equivalent sets. If \code{TRUE}, labels are constructed using the codes valid at
the latest provided date, and comma separated like in the example below. By
default, labels will be placed in a column named \code{label}.

\if{html}{\out{<div class="sourceCode">}}\preformatted{"1508 Ålesund, 1580 Haram"
}\if{html}{\out{</div>}}

If \code{FALSE}, no labels will be applied.

This parameter also accepts a named list of labelling functions. The
resulting dataset will contain a label column for each of the supplied
functions. The names of the label columns are specified using the names of
the list of functions.

The functions provided in this parameter can accept any of the following
parameters: \code{date}, \code{code}, \code{name}, \code{validFrom} and \code{validTo}, representing
the corresponding values of each code in a group. The functions must also
provide a \code{...} parameter, unless using all of the above. The functions can
expect that the input variables have the same length of 1 or longer. The
functions should return a character vector of length one or the same length
as the input variables.

The following list, when supplied to this parameter, creates two label
columns: one containing the codes and names (\code{label1}), and another with
only the codes \code{label2}. In this example, \code{label1} creates the same labels
as the default labelling used when \code{labels == TRUE}.

\if{html}{\out{<div class="sourceCode">}}\preformatted{list(
  label1 = function(code, name, date, ...) \{
    label_codes <- date == max(date)
    paste(code[label_codes], name[label_codes], collapse = ", ")
  \},
  label2 = function(code, date, ...) \{
    label_codes <- date == max(date)
    paste(code[label_codes], collapse = ", ")
  \}
)
}\if{html}{\out{</div>}}}

\item{graph}{Optional. Generating the graph using \code{klass_graph} manually
beforehand and providing it in this parameter can save time if running
\code{find_equivalent_codes} multiple times in sequence.}

\item{date_format}{Optional. Passed directly to \link{format}, this is used to
specify the output format for the \code{date} column. The default keeps just the
year (\code{"YYYY"}). To get the full date in \code{"YYYY-MM-DD"} format, use
\code{"\%Y-\%m-\%d"}. See \link{strptime} for complete functionality.}
}
\value{
A data.frame with columns:
\itemize{
\item \code{date} containing the input \code{dates}
\item \code{code} containing the set of equivalent codes in each date
\item \code{name} containing the names of each code
\item \code{validFrom} and \code{validTo} values for each code returned
\item By default, \code{labels}, giving a unique group label for each group of equivalent sets.
}
}
\description{
Find which codes should be combined to reconstruct older or newer codes
across multiple dates. Creates groups dynamically to fit the provided date
range, with flexible labelling functionality.
}
\details{
This function provides a solution to the problem of split or
combined codes in Klass classifications. When using \code{\link{update_klass}} to ask
"what is this code in this version of the classification in this other
version of the classification?", the answer is sometimes that the code has
been split into two or more codes (or combined from two or more codes, if
trying to back-date a code), and therefore that the code cannot be updated.

The solution provided by \code{find_equivalent_codes} is answering the question: "in
these versions of the classification, which codes were equivalent to this
code in this other version of the classification?".

Consider the following example of two codes combining into one. Here, \code{"a"}
and \code{"b"} are valid at t1, and are combined into \code{"c"} at t2.

\if{html}{\out{<div class="sourceCode">}}\preformatted{t1      t2
a ---|
b ---+-> c

}\if{html}{\out{</div>}}

\code{\link{update_klass}} would inform us that \code{"a"} can be updated to
\code{"c"} at t2, unless we specified \code{combine = FALSE}, in which case the
result would be \code{NA}. \code{find_equivalent_codes()} would inform us that the
equivalent of the codes \code{"a"} and \code{"b"} in t1 at t2 is \code{"c"}.

We can also consider a code splitting into two. In this example, \code{"a"} is
valid at t1, and splits into \code{"b"} and \code{"c"} at t2.

\if{html}{\out{<div class="sourceCode">}}\preformatted{t1     t2
a
|-----> b
|-----> c
}\if{html}{\out{</div>}}

\code{\link{update_klass}} is unable to provide an updated code due to the
split, and would return \code{NA}. \code{find_equivalent_codes} would inform us that the
equivalent codes of \code{"a"} at t1 is \code{"b"} and \code{"c"} at t2.

\code{find_equivalent_codes} can handle more than two dates. In the following
example, \code{"a"} splits into \code{"b"} and \code{"c"} at t2, and \code{"b"} and \code{"c"}
combine into \code{"d"} at t3. \code{find_equivalent_codes} can inform us that \code{"a"} is
equivalent to \code{"b"} and \code{"c"} at t2, and \code{"d"} at t3.

\if{html}{\out{<div class="sourceCode">}}\preformatted{t1     t2     t3
a
|-----> b |
|-----> c +-> d
}\if{html}{\out{</div>}}

\code{find_equivalent_codes} will only search in the time range we specify. As a
consequence, generating sets of equivalent codes over longer time spans
will generally create larger sets than using shorter time spans.

To illustrate this behavior, we can add a new code \code{"e"} to the previous
example, and have \code{"d"} and \code{"e"} combine into \code{"f"} at t4.

\if{html}{\out{<div class="sourceCode">}}\preformatted{t1     t2     t3     t4
a
|-----> b |
|-----> c |-> d |
e---------------+--> f
}\if{html}{\out{</div>}}

Finding the
equivalents of \code{"a"} in t1 at t2 and t3 returns the same sets as
before:
\itemize{
\item t1: \code{"a"}
\item t2: \code{"b"} and \code{"c"}
\item t3: \code{"d"}
}

However, if we also wanted to know the equivalent set for t4, the result
would be:
\itemize{
\item t1: \code{"a"} and \code{"e"}
\item t2: \code{"b"}, \code{"c"} and \code{"e"}
\item t3: \code{"d"} and \code{"e"}
\item t4: \code{"f"}
}

\code{find_equivalents} is a legacy alias for \code{find_equivalent_codes}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format.R
\name{formattering_kommune}
\alias{formattering_kommune}
\title{Format vector for kommune codes}
\usage{
formattering_kommune(x)
}
\arguments{
\item{x}{\itemize{
\item vector of character type for kommune codes
}}
}
\value{
vector of character
}
\description{
Format vector for kommune codes
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Hent_data.R
\name{GetUrl2}
\alias{GetUrl2}
\title{Get json file from Url - alternative version}
\usage{
GetUrl2(url, check = TRUE)
}
\arguments{
\item{url}{String url address}

\item{check}{Logical parameter on whether to check if the url exists}
}
\value{
text in json format
}
\description{
Get json file from Url - alternative version
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Klass_list.R
\name{search_klass}
\alias{search_klass}
\alias{SearchKlass}
\title{Search Klass}
\usage{
search_klass(query, codelists = FALSE, size = 20)

SearchKlass(query, codelists = FALSE, size = 20)
}
\arguments{
\item{query}{String with key word to search for}

\item{codelists}{True/False for whether to include codelists. Default = FALSE}

\item{size}{The number of results to show. Default = 20.}
}
\value{
Data frame of possible classifications that match the query
}
\description{
Search Klass
}
\examples{
search_klass("occupation")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Klass_list.R
\name{list_family}
\alias{list_family}
\alias{ListFamily}
\title{Classification family list
Print a list of all families and the number of classifications in each}
\usage{
list_family(family = NULL, codelists = FALSE, language = "nb")

ListFamily(family = NULL, codelists = FALSE, language = "nb")
}
\arguments{
\item{family}{Input family ID number to get a list of classifications in that family}

\item{codelists}{True/False for whether to include codelists. Default = FALSE}

\item{language}{Two letter string for the requested language output. Default is Bokmål ("nb"). Nynorsk ("nn") and English ("en").}
}
\value{
dataset containing a list of families
}
\description{
Classification family list
Print a list of all families and the number of classifications in each
}
\examples{
list_family(family = 1)
}
